# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Module dedicated functions/classes dealing with rate limiting requests.
"""

import collections
import copy
import httplib
import math
import re
import time

import webob.dec
import webob.exc

from vsm.api.openstack import wsgi
from vsm.api.views import limits as limits_views
from vsm.api import xmlutil
from vsm.openstack.common import importutils
from vsm.openstack.common import jsonutils
from vsm import quota
from vsm import wsgi as base_wsgi

QUOTAS = quota.QUOTAS

# Convenience constants for the limits dictionary passed to Limiter().
PER_SECOND = 1
PER_MINUTE = 60
PER_HOUR = 60 * 60
PER_DAY = 60 * 60 * 24

limits_nsmap = {None: xmlutil.XMLNS_COMMON_V10, 'atom': xmlutil.XMLNS_ATOM}

class LimitsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('limits', selector='limits')

        rates = xmlutil.SubTemplateElement(root, 'rates')
        rate = xmlutil.SubTemplateElement(rates, 'rate', selector='rate')
        rate.set('uri', 'uri')
        rate.set('regex', 'regex')
        limit = xmlutil.SubTemplateElement(rate, 'limit', selector='limit')
        limit.set('value', 'value')
        limit.set('verb', 'verb')
        limit.set('remaining', 'remaining')
        limit.set('unit', 'unit')
        limit.set('next-available', 'next-available')

        absolute = xmlutil.SubTemplateElement(root, 'absolute',
                                              selector='absolute')
        limit = xmlutil.SubTemplateElement(absolute, 'limit',
                                           selector=xmlutil.get_items)
        limit.set('name', 0)
        limit.set('value', 1)

        return xmlutil.MasterTemplate(root, 1, nsmap=limits_nsmap)

class LimitsController(object):
    """
    Controller for accessing limits in the OpenStack API.
    """

    @wsgi.serializers(xml=LimitsTemplate)
    def index(self, req):
        """
        Return all global and rate limit information.
        """
        context = req.environ['vsm.context']
        quotas = QUOTAS.get_project_quotas(context, context.project_id,
                                           usages=False)
        abs_limits = dict((k, v['limit']) for k, v in quotas.items())
        rate_limits = req.environ.get("vsm.limits", [])

        builder = self._get_view_builder(req)
        return builder.build(rate_limits, abs_limits)

    def _get_view_builder(self, req):
        return limits_views.ViewBuilder()

def create_resource():
    return wsgi.Resource(LimitsController())

class Limit(object):
    """
    Stores information about a limit for HTTP requests.
    """

    UNITS = {
        1: "SECOND",
        60: "MINUTE",
        60 * 60: "HOUR",
        60 * 60 * 24: "DAY",
    }

    UNIT_MAP = dict([(v, k) for k, v in UNITS.items()])

    def __init__(self, verb, uri, regex, value, unit):
        """
        Initialize a new `Limit`.

        @param verb: HTTP verb (POST, PUT, etc.)
        @param uri: Human-readable URI
        @param regex: Regular expression format for this limit
        @param value: Integer number of requests which can be made
        @param unit: Unit of measure for the value parameter
        """
        self.verb = verb
        self.uri = uri
        self.regex = regex
        self.value = int(value)
        self.unit = unit
        self.unit_string = self.display_unit().lower()
        self.remaining = int(value)

        if value <= 0:
            raise ValueError("Limit value must be > 0")

        self.last_request = None
        self.next_request = None

        self.water_level = 0
        self.capacity = self.unit
        self.request_value = float(self.capacity) / float(self.value)
        msg = _("Only %(value)s %(verb)s request(s) can be "
                "made to %(uri)s every %(unit_string)s.")
        self.error_message = msg % self.__dict__

    def __call__(self, verb, url):
        """
        Represents a call to this limit from a relevant request.

        @param verb: string http verb (POST, GET, etc.)
        @param url: string URL
        """
        if self.verb != verb or not re.match(self.regex, url):
            return

        now = self._get_time()

        if self.last_request is None:
            self.last_request = now

        leak_value = now - self.last_request

        self.water_level -= leak_value
        self.water_level = max(self.water_level, 0)
        self.water_level += self.request_value

        difference = self.water_level - self.capacity

        self.last_request = now

        if difference > 0:
            self.water_level -= self.request_value
            self.next_request = now + difference
            return difference

        cap = self.capacity
        water = self.water_level
        val = self.value

        self.remaining = math.floor(((cap - water) / cap) * val)
        self.next_request = now

    def _get_time(self):
        """Retrieve the current time. Broken out for testability."""
        return time.time()

    def display_unit(self):
        """Display the string name of the unit."""
        return self.UNITS.get(self.unit, "UNKNOWN")

    def display(self):
        """Return a useful representation of this class."""
        return {
            "verb": self.verb,
            "URI": self.uri,
            "regex": self.regex,
            "value": self.value,
            "remaining": int(self.remaining),
            "unit": self.display_unit(),
            "resetTime": int(self.next_request or self._get_time()),
        }

# "Limit" format is a dictionary with the HTTP verb, human-readable URI,
# a regular-expression to match, value and unit of measure (PER_DAY, etc.)

DEFAULT_LIMITS = [
    Limit("POST", "*", ".*", 10, PER_MINUTE),
    Limit("POST", "*/servers", "^/servers", 50, PER_DAY),
    Limit("PUT", "*", ".*", 10, PER_MINUTE),
    Limit("GET", "*changes-since*", ".*changes-since.*", 3, PER_MINUTE),
    Limit("DELETE", "*", ".*", 100, PER_MINUTE),
]

class RateLimitingMiddleware(base_wsgi.Middleware):
    """
    Rate-limits requests passing through this middleware. All limit information
    is stored in memory for this implementation.
    """

    def __init__(self, application, limits=None, limiter=None, **kwargs):
        """
        Initialize new `RateLimitingMiddleware`, which wraps the given WSGI
        application and sets up the given limits.

        @param application: WSGI application to wrap
        @param limits: String describing limits
        @param limiter: String identifying class for representing limits

        Other parameters are passed to the constructor for the limiter.
        """
        base_wsgi.Middleware.__init__(self, application)

        # Select the limiter class
        if limiter is None:
            limiter = Limiter
        else:
            limiter = importutils.import_class(limiter)

        # Parse the limits, if any are provided
        if limits is not None:
            limits = limiter.parse_limits(limits)

        self._limiter = limiter(limits or DEFAULT_LIMITS, **kwargs)

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        """
        Represents a single call through this middleware. We should record the
        request if we have a limit relevant to it. If no limit is relevant to
        the request, ignore it.

        If the request should be rate limited, return a fault telling the user
        they are over the limit and need to retry later.
        """
        verb = req.method
        url = req.url
        context = req.environ.get("vsm.context")

        if context:
            username = context.user_id
        else:
            username = None

        delay, error = self._limiter.check_for_delay(verb, url, username)

        if delay:
            msg = _("This request was rate-limited.")
            retry = time.time() + delay
            return wsgi.OverLimitFault(msg, error, retry)

        req.environ["vsm.limits"] = self._limiter.get_limits(username)

        return self.application

class Limiter(object):
    """
    Rate-limit checking class which handles limits in memory.
    """

    def __init__(self, limits, **kwargs):
        """
        Initialize the new `Limiter`.

        @param limits: List of `Limit` objects
        """
        self.limits = copy.deepcopy(limits)
        self.levels = collections.defaultdict(lambda: copy.deepcopy(limits))

        # Pick up any per-user limit information
        for key, value in kwargs.items():
            if key.startswith('user:'):
                username = key[5:]
                self.levels[username] = self.parse_limits(value)

    def get_limits(self, username=None):
        """
        Return the limits for a given user.
        """
        return [limit.display() for limit in self.levels[username]]

    def check_for_delay(self, verb, url, username=None):
        """
        Check the given verb/user/user triplet for limit.

        @return: Tuple of delay (in seconds) and error message (or None, None)
        """
        delays = []

        for limit in self.levels[username]:
            delay = limit(verb, url)
            if delay:
                delays.append((delay, limit.error_message))

        if delays:
            delays.sort()
            return delays[0]

        return None, None

    # Note: This method gets called before the class is instantiated,
    # so this must be either a static method or a class method.  It is
    # used to develop a list of limits to feed to the constructor.  We
    # put this in the class so that subclasses can override the
    # default limit parsing.
    @staticmethod
    def parse_limits(limits):
        """
        Convert a string into a list of Limit instances.  This
        implementation expects a semicolon-separated sequence of
        parenthesized groups, where each group contains a
        comma-separated sequence consisting of HTTP method,
        user-readable URI, a URI reg-exp, an integer number of
        requests which can be made, and a unit of measure.  Valid
        values for the latter are "SECOND", "MINUTE", "HOUR", and
        "DAY".

        @return: List of Limit instances.
        """

        # Handle empty limit strings
        limits = limits.strip()
        if not limits:
            return []

        # Split up the limits by semicolon
        result = []
        for group in limits.split(';'):
            group = group.strip()
            if group[:1] != '(' or group[-1:] != ')':
                raise ValueError("Limit rules must be surrounded by "
                                 "parentheses")
            group = group[1:-1]

            # Extract the Limit arguments
            args = [a.strip() for a in group.split(',')]
            if len(args) != 5:
                raise ValueError("Limit rules must contain the following "
                                 "arguments: verb, uri, regex, value, unit")

            # Pull out the arguments
            verb, uri, regex, value, unit = args

            # Upper-case the verb
            verb = verb.upper()

            # Convert value--raises ValueError if it's not integer
            value = int(value)

            # Convert unit
            unit = unit.upper()
            if unit not in Limit.UNIT_MAP:
                raise ValueError("Invalid units specified")
            unit = Limit.UNIT_MAP[unit]

            # Build a limit
            result.append(Limit(verb, uri, regex, value, unit))

        return result

class WsgiLimiter(object):
    """
    Rate-limit checking from a WSGI application. Uses an in-memory `Limiter`.

    To use, POST ``/<username>`` with JSON data such as::

        {
            "verb" : GET,
            "path" : "/servers"
        }

    and receive a 204 No Content, or a 403 Forbidden with an X-Wait-Seconds
    header containing the number of seconds to wait before the action would
    succeed.
    """

    def __init__(self, limits=None):
        """
        Initialize the new `WsgiLimiter`.

        @param limits: List of `Limit` objects
        """
        self._limiter = Limiter(limits or DEFAULT_LIMITS)

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, request):
        """
        Handles a call to this application. Returns 204 if the request is
        acceptable to the limiter, else a 403 is returned with a relevant
        header indicating when the request *will* succeed.
        """
        if request.method != "POST":
            raise webob.exc.HTTPMethodNotAllowed()

        try:
            info = dict(jsonutils.loads(request.body))
        except ValueError:
            raise webob.exc.HTTPBadRequest()

        username = request.path_info_pop()
        verb = info.get("verb")
        path = info.get("path")

        delay, error = self._limiter.check_for_delay(verb, path, username)

        if delay:
            headers = {"X-Wait-Seconds": "%.2f" % delay}
            return webob.exc.HTTPForbidden(headers=headers, explanation=error)
        else:
            return webob.exc.HTTPNoContent()

class WsgiLimiterProxy(object):
    """
    Rate-limit requests based on answers from a remote source.
    """

    def __init__(self, limiter_address):
        """
        Initialize the new `WsgiLimiterProxy`.

        @param limiter_address: IP/port combination of where to request limit
        """
        self.limiter_address = limiter_address

    def check_for_delay(self, verb, path, username=None):
        body = jsonutils.dumps({"verb": verb, "path": path})
        headers = {"Content-Type": "application/json"}

        conn = httplib.HTTPConnection(self.limiter_address)

        if username:
            conn.request("POST", "/%s" % (username), body, headers)
        else:
            conn.request("POST", "/", body, headers)

        resp = conn.getresponse()

        if 200 >= resp.status < 300:
            return None, None

        return resp.getheader("X-Wait-Seconds"), resp.read() or None

    # Note: This method gets called before the class is instantiated,
    # so this must be either a static method or a class method.  It is
    # used to develop a list of limits to feed to the constructor.
    # This implementation returns an empty list, since all limit
    # decisions are made by a remote server.
    @staticmethod
    def parse_limits(limits):
        """
        Ignore a limits string--simply doesn't apply for the limit
        proxy.

        @return: Empty list.
        """

        return []



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 OpenStack LLC.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""RequestContext: context for requests that persist through all of vsm."""

import copy
import uuid

from vsm.openstack.common import local
from vsm.openstack.common import log as logging
from vsm.openstack.common import timeutils
from vsm import policy

LOG = logging.getLogger(__name__)

def generate_request_id():
    return 'req-' + str(uuid.uuid4())

class RequestContext(object):
    """Security context and request information.

    Represents the user taking a given action within the system.

    """

    def __init__(self, user_id, project_id, is_admin=None, read_deleted="no",
                 roles=None, remote_address=None, timestamp=None,
                 request_id=None, auth_token=None, overwrite=True,
                 quota_class=None, **kwargs):
        """
        :param read_deleted: 'no' indicates deleted records are hidden, 'yes'
            indicates deleted records are visible, 'only' indicates that
            *only* deleted records are visible.

        :param overwrite: Set to False to ensure that the greenthread local
            copy of the index is not overwritten.

        :param kwargs: Extra arguments that might be present, but we ignore
            because they possibly came in from older rpc messages.
        """
        if kwargs:
            LOG.warn(_('Arguments dropped when creating context: %s') %
                     str(kwargs))
        self.user_id = user_id
        self.project_id = project_id
        self.roles = roles or []
        self.is_admin = is_admin
        if self.is_admin is None:
            self.is_admin = policy.check_is_admin(self.roles)
        elif self.is_admin and 'admin' not in self.roles:
            self.roles.append('admin')
        self.read_deleted = read_deleted
        self.remote_address = remote_address
        if not timestamp:
            timestamp = timeutils.utcnow()
        if isinstance(timestamp, basestring):
            timestamp = timeutils.parse_strtime(timestamp)
        self.timestamp = timestamp
        if not request_id:
            request_id = generate_request_id()
        self.request_id = request_id
        self.auth_token = auth_token
        self.quota_class = quota_class
        if overwrite or not hasattr(local.store, 'context'):
            self.update_store()
        # This is for auto report procedure status in vsm.
        # Plz check utils.py status_checking filter function.
        self.status_values = None

    def get_status_values(self):
        return self.status_values

    def set_status_values(self, parameters):
        self.status_values = parameters

    def _get_read_deleted(self):
        return self._read_deleted

    def _set_read_deleted(self, read_deleted):
        if read_deleted not in ('no', 'yes', 'only'):
            raise ValueError(_("read_deleted can only be one of 'no', "
                               "'yes' or 'only', not %r") % read_deleted)
        self._read_deleted = read_deleted

    def _del_read_deleted(self):
        del self._read_deleted

    read_deleted = property(_get_read_deleted, _set_read_deleted,
                            _del_read_deleted)

    def update_store(self):
        local.store.context = self

    def to_dict(self):
        return {'user_id': self.user_id,
                'project_id': self.project_id,
                'is_admin': self.is_admin,
                'read_deleted': self.read_deleted,
                'roles': self.roles,
                'remote_address': self.remote_address,
                'timestamp': timeutils.strtime(self.timestamp),
                'request_id': self.request_id,
                'auth_token': self.auth_token,
                'quota_class': self.quota_class,
                'tenant': self.tenant,
                'user': self.user,
                'status_values': self.status_values}

    @classmethod
    def from_dict(cls, values):
        return cls(**values)

    def elevated(self, read_deleted=None, overwrite=False):
        """Return a version of this context with admin flag set."""
        context = copy.copy(self)
        context.is_admin = True

        if 'admin' not in context.roles:
            context.roles.append('admin')

        if read_deleted is not None:
            context.read_deleted = read_deleted

        return context

    # NOTE(sirp): the openstack/common version of RequestContext uses
    # tenant/user whereas the Vsm version uses project_id/user_id. We need
    # this shim in order to use context-aware code from openstack/common, like
    # logging, until we make the switch to using openstack/common's version of
    # RequestContext.
    @property
    def tenant(self):
        return self.project_id

    @property
    def user(self):
        return self.user_id

def get_admin_context(read_deleted="no"):
    return RequestContext(user_id=None,
                          project_id=None,
                          is_admin=True,
                          read_deleted=read_deleted,
                          overwrite=False)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

#    Copyright 2011 Cloudscaling Group, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pprint
import re
import socket
import sys
import types
import uuid

import eventlet
import greenlet
from oslo.config import cfg

from vsm.openstack.common import excutils
from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import importutils
from vsm.openstack.common import jsonutils
from vsm.openstack.common import processutils as utils
from vsm.openstack.common.rpc import common as rpc_common

zmq = importutils.try_import('eventlet.green.zmq')

# for convenience, are not modified.
pformat = pprint.pformat
Timeout = eventlet.timeout.Timeout
LOG = rpc_common.LOG
RemoteError = rpc_common.RemoteError
RPCException = rpc_common.RPCException

zmq_opts = [
    cfg.StrOpt('rpc_zmq_bind_address', default='*',
               help='ZeroMQ bind address. Should be a wildcard (*), '
                    'an ethernet interface, or IP. '
                    'The "host" option should point or resolve to this '
                    'address.'),

    # The module.Class to use for matchmaking.
    cfg.StrOpt(
        'rpc_zmq_matchmaker',
        default=('vsm.openstack.common.rpc.'
                 'matchmaker.MatchMakerLocalhost'),
        help='MatchMaker driver',
    ),

    # The following port is unassigned by IANA as of 2012-05-21
    cfg.IntOpt('rpc_zmq_port', default=9501,
               help='ZeroMQ receiver listening port'),

    cfg.IntOpt('rpc_zmq_contexts', default=1,
               help='Number of ZeroMQ contexts, defaults to 1'),

    cfg.IntOpt('rpc_zmq_topic_backlog', default=None,
               help='Maximum number of ingress messages to locally buffer '
                    'per topic. Default is unlimited.'),

    cfg.StrOpt('rpc_zmq_ipc_dir', default='/var/run/openstack',
               help='Directory for holding IPC sockets'),

    cfg.StrOpt('rpc_zmq_host', default=socket.gethostname(),
               help='Name of this node. Must be a valid hostname, FQDN, or '
                    'IP address. Must match "host" option, if running Nova.')
]

CONF = cfg.CONF
CONF.register_opts(zmq_opts)

ZMQ_CTX = None  # ZeroMQ Context, must be global.
matchmaker = None  # memoized matchmaker object

def _serialize(data):
    """
    Serialization wrapper
    We prefer using JSON, but it cannot encode all types.
    Error if a developer passes us bad data.
    """
    try:
        return jsonutils.dumps(data, ensure_ascii=True)
    except TypeError:
        with excutils.save_and_reraise_exception():
            LOG.error(_("JSON serialization failed."))

def _deserialize(data):
    """
    Deserialization wrapper
    """
    LOG.debug(_("Deserializing: %s"), data)
    return jsonutils.loads(data)

class ZmqSocket(object):
    """
    A tiny wrapper around ZeroMQ to simplify the send/recv protocol
    and connection management.

    Can be used as a Context (supports the 'with' statement).
    """

    def __init__(self, addr, zmq_type, bind=True, subscribe=None):
        self.sock = _get_ctxt().socket(zmq_type)
        self.addr = addr
        self.type = zmq_type
        self.subscriptions = []

        # Support failures on sending/receiving on wrong socket type.
        self.can_recv = zmq_type in (zmq.PULL, zmq.SUB)
        self.can_send = zmq_type in (zmq.PUSH, zmq.PUB)
        self.can_sub = zmq_type in (zmq.SUB, )

        # Support list, str, & None for subscribe arg (cast to list)
        do_sub = {
            list: subscribe,
            str: [subscribe],
            type(None): []
        }[type(subscribe)]

        for f in do_sub:
            self.subscribe(f)

        str_data = {'addr': addr, 'type': self.socket_s(),
                    'subscribe': subscribe, 'bind': bind}

        LOG.debug(_("Connecting to %(addr)s with %(type)s"), str_data)
        LOG.debug(_("-> Subscribed to %(subscribe)s"), str_data)
        LOG.debug(_("-> bind: %(bind)s"), str_data)

        try:
            if bind:
                self.sock.bind(addr)
            else:
                self.sock.connect(addr)
        except Exception:
            raise RPCException(_("Could not open socket."))

    def socket_s(self):
        """Get socket type as string."""
        t_enum = ('PUSH', 'PULL', 'PUB', 'SUB', 'REP', 'REQ', 'ROUTER',
                  'DEALER')
        return dict(map(lambda t: (getattr(zmq, t), t), t_enum))[self.type]

    def subscribe(self, msg_filter):
        """Subscribe."""
        if not self.can_sub:
            raise RPCException("Cannot subscribe on this socket.")
        LOG.debug(_("Subscribing to %s"), msg_filter)

        try:
            self.sock.setsockopt(zmq.SUBSCRIBE, msg_filter)
        except Exception:
            return

        self.subscriptions.append(msg_filter)

    def unsubscribe(self, msg_filter):
        """Unsubscribe."""
        if msg_filter not in self.subscriptions:
            return
        self.sock.setsockopt(zmq.UNSUBSCRIBE, msg_filter)
        self.subscriptions.remove(msg_filter)

    def close(self):
        if self.sock is None or self.sock.closed:
            return

        # We must unsubscribe, or we'll leak descriptors.
        if len(self.subscriptions) > 0:
            for f in self.subscriptions:
                try:
                    self.sock.setsockopt(zmq.UNSUBSCRIBE, f)
                except Exception:
                    pass
            self.subscriptions = []

        try:
            # Default is to linger
            self.sock.close()
        except Exception:
            # While this is a bad thing to happen,
            # it would be much worse if some of the code calling this
            # were to fail. For now, lets log, and later evaluate
            # if we can safely raise here.
            LOG.error("ZeroMQ socket could not be closed.")
        self.sock = None

    def recv(self):
        if not self.can_recv:
            raise RPCException(_("You cannot recv on this socket."))
        return self.sock.recv_multipart()

    def send(self, data):
        if not self.can_send:
            raise RPCException(_("You cannot send on this socket."))
        self.sock.send_multipart(data)

class ZmqClient(object):
    """Client for ZMQ sockets."""

    def __init__(self, addr, socket_type=None, bind=False):
        if socket_type is None:
            socket_type = zmq.PUSH
        self.outq = ZmqSocket(addr, socket_type, bind=bind)

    def cast(self, msg_id, topic, data, envelope=False):
        msg_id = msg_id or 0

        if not (envelope or rpc_common._SEND_RPC_ENVELOPE):
            self.outq.send(map(bytes,
                           (msg_id, topic, 'cast', _serialize(data))))
            return

        rpc_envelope = rpc_common.serialize_msg(data[1], envelope)
        zmq_msg = reduce(lambda x, y: x + y, rpc_envelope.items())
        self.outq.send(map(bytes,
                       (msg_id, topic, 'impl_zmq_v2', data[0]) + zmq_msg))

    def close(self):
        self.outq.close()

class RpcContext(rpc_common.CommonRpcContext):
    """Context that supports replying to a rpc.call."""
    def __init__(self, **kwargs):
        self.replies = []
        super(RpcContext, self).__init__(**kwargs)

    def deepcopy(self):
        values = self.to_dict()
        values['replies'] = self.replies
        return self.__class__(**values)

    def reply(self, reply=None, failure=None, ending=False):
        if ending:
            return
        self.replies.append(reply)

    @classmethod
    def marshal(self, ctx):
        ctx_data = ctx.to_dict()
        return _serialize(ctx_data)

    @classmethod
    def unmarshal(self, data):
        return RpcContext.from_dict(_deserialize(data))

class InternalContext(object):
    """Used by ConsumerBase as a private context for - methods."""

    def __init__(self, proxy):
        self.proxy = proxy
        self.msg_waiter = None

    def _get_response(self, ctx, proxy, topic, data):
        """Process a curried message and cast the result to topic."""
        LOG.debug(_("Running func with context: %s"), ctx.to_dict())
        data.setdefault('version', None)
        data.setdefault('args', {})

        try:
            result = proxy.dispatch(
                ctx, data['version'], data['method'], **data['args'])
            return ConsumerBase.normalize_reply(result, ctx.replies)
        except greenlet.GreenletExit:
            # ignore these since they are just from shutdowns
            pass
        except rpc_common.ClientException, e:
            LOG.debug(_("Expected exception during message handling (%s)") %
                      e._exc_info[1])
            return {'exc':
                    rpc_common.serialize_remote_exception(e._exc_info,
                                                          log_failure=False)}
        except Exception:
            LOG.error(_("Exception during message handling"))
            return {'exc':
                    rpc_common.serialize_remote_exception(sys.exc_info())}

    def reply(self, ctx, proxy,
              msg_id=None, context=None, topic=None, msg=None):
        """Reply to a casted call."""
        # Our real method is curried into msg['args']

        child_ctx = RpcContext.unmarshal(msg[0])
        response = ConsumerBase.normalize_reply(
            self._get_response(child_ctx, proxy, topic, msg[1]),
            ctx.replies)

        LOG.debug(_("Sending reply"))
        _multi_send(_cast, ctx, topic, {
            'method': '-process_reply',
            'args': {
                'msg_id': msg_id,  # Include for Folsom compat.
                'response': response
            }
        }, _msg_id=msg_id)

class ConsumerBase(object):
    """Base Consumer."""

    def __init__(self):
        self.private_ctx = InternalContext(None)

    @classmethod
    def normalize_reply(self, result, replies):
        #TODO(ewindisch): re-evaluate and document this method.
        if isinstance(result, types.GeneratorType):
            return list(result)
        elif replies:
            return replies
        else:
            return [result]

    def process(self, proxy, ctx, data):
        data.setdefault('version', None)
        data.setdefault('args', {})

        # Method starting with - are
        # processed internally. (non-valid method name)
        method = data.get('method')
        if not method:
            LOG.error(_("RPC message did not include method."))
            return

        # Internal method
        # uses internal context for safety.
        if method == '-reply':
            self.private_ctx.reply(ctx, proxy, **data['args'])
            return

        proxy.dispatch(ctx, data['version'],
                       data['method'], **data['args'])

class ZmqBaseReactor(ConsumerBase):
    """
    A consumer class implementing a
    centralized casting broker (PULL-PUSH)
    for RoundRobin requests.
    """

    def __init__(self, conf):
        super(ZmqBaseReactor, self).__init__()

        self.mapping = {}
        self.proxies = {}
        self.threads = []
        self.sockets = []
        self.subscribe = {}

        self.pool = eventlet.greenpool.GreenPool(conf.rpc_thread_pool_size)

    def register(self, proxy, in_addr, zmq_type_in, out_addr=None,
                 zmq_type_out=None, in_bind=True, out_bind=True,
                 subscribe=None):

        LOG.info(_("Registering reactor"))

        if zmq_type_in not in (zmq.PULL, zmq.SUB):
            raise RPCException("Bad input socktype")

        # Items push in.
        inq = ZmqSocket(in_addr, zmq_type_in, bind=in_bind,
                        subscribe=subscribe)

        self.proxies[inq] = proxy
        self.sockets.append(inq)

        LOG.info(_("In reactor registered"))

        if not out_addr:
            return

        if zmq_type_out not in (zmq.PUSH, zmq.PUB):
            raise RPCException("Bad output socktype")

        # Items push out.
        outq = ZmqSocket(out_addr, zmq_type_out, bind=out_bind)

        self.mapping[inq] = outq
        self.mapping[outq] = inq
        self.sockets.append(outq)

        LOG.info(_("Out reactor registered"))

    def consume_in_thread(self):
        def _consume(sock):
            LOG.info(_("Consuming socket"))
            while True:
                self.consume(sock)

        for k in self.proxies.keys():
            self.threads.append(
                self.pool.spawn(_consume, k)
            )

    def wait(self):
        for t in self.threads:
            t.wait()

    def close(self):
        for s in self.sockets:
            s.close()

        for t in self.threads:
            t.kill()

class ZmqProxy(ZmqBaseReactor):
    """
    A consumer class implementing a
    topic-based proxy, forwarding to
    IPC sockets.
    """

    def __init__(self, conf):
        super(ZmqProxy, self).__init__(conf)
        pathsep = set((os.path.sep or '', os.path.altsep or '', '/', '\\'))
        self.badchars = re.compile(r'[%s]' % re.escape(''.join(pathsep)))

        self.topic_proxy = {}

    def consume(self, sock):
        ipc_dir = CONF.rpc_zmq_ipc_dir

        #TODO(ewindisch): use zero-copy (i.e. references, not copying)
        data = sock.recv()
        topic = data[1]

        LOG.debug(_("CONSUMER GOT %s"), ' '.join(map(pformat, data)))

        if topic.startswith('fanout~'):
            sock_type = zmq.PUB
            topic = topic.split('.', 1)[0]
        elif topic.startswith('zmq_replies'):
            sock_type = zmq.PUB
        else:
            sock_type = zmq.PUSH

        if topic not in self.topic_proxy:
            def publisher(waiter):
                LOG.info(_("Creating proxy for topic: %s"), topic)

                try:
                    # The topic is received over the network,
                    # don't trust this input.
                    if self.badchars.search(topic) is not None:
                        emsg = _("Topic contained dangerous characters.")
                        LOG.warn(emsg)
                        raise RPCException(emsg)

                    out_sock = ZmqSocket("ipc://%s/zmq_topic_%s" %
                                         (ipc_dir, topic),
                                         sock_type, bind=True)
                except RPCException:
                    waiter.send_exception(*sys.exc_info())
                    return

                self.topic_proxy[topic] = eventlet.queue.LightQueue(
                    CONF.rpc_zmq_topic_backlog)
                self.sockets.append(out_sock)

                # It takes some time for a pub socket to open,
                # before we can have any faith in doing a send() to it.
                if sock_type == zmq.PUB:
                    eventlet.sleep(.5)

                waiter.send(True)

                while(True):
                    data = self.topic_proxy[topic].get()
                    out_sock.send(data)
                    LOG.debug(_("ROUTER RELAY-OUT SUCCEEDED %(data)s") %
                              {'data': data})

            wait_sock_creation = eventlet.event.Event()
            eventlet.spawn(publisher, wait_sock_creation)

            try:
                wait_sock_creation.wait()
            except RPCException:
                LOG.error(_("Topic socket file creation failed."))
                return

        try:
            self.topic_proxy[topic].put_nowait(data)
            LOG.debug(_("ROUTER RELAY-OUT QUEUED %(data)s") %
                      {'data': data})
        except eventlet.queue.Full:
            LOG.error(_("Local per-topic backlog buffer full for topic "
                        "%(topic)s. Dropping message.") % {'topic': topic})

    def consume_in_thread(self):
        """Runs the ZmqProxy service"""
        ipc_dir = CONF.rpc_zmq_ipc_dir
        consume_in = "tcp://%s:%s" % \
            (CONF.rpc_zmq_bind_address,
             CONF.rpc_zmq_port)
        consumption_proxy = InternalContext(None)

        if not os.path.isdir(ipc_dir):
            try:
                utils.execute('mkdir', '-p', ipc_dir, run_as_root=True)
                utils.execute('chown', "%s:%s" % (os.getuid(), os.getgid()),
                              ipc_dir, run_as_root=True)
                utils.execute('chmod', '750', ipc_dir, run_as_root=True)
            except utils.ProcessExecutionError:
                with excutils.save_and_reraise_exception():
                    LOG.error(_("Could not create IPC directory %s") %
                              (ipc_dir, ))

        try:
            self.register(consumption_proxy,
                          consume_in,
                          zmq.PULL,
                          out_bind=True)
        except zmq.ZMQError:
            with excutils.save_and_reraise_exception():
                LOG.error(_("Could not create ZeroMQ receiver daemon. "
                            "Socket may already be in use."))

        super(ZmqProxy, self).consume_in_thread()

def unflatten_envelope(packenv):
    """Unflattens the RPC envelope.
       Takes a list and returns a dictionary.
       i.e. [1,2,3,4] => {1: 2, 3: 4}
    """
    i = iter(packenv)
    h = {}
    try:
        while True:
            k = i.next()
            h[k] = i.next()
    except StopIteration:
        return h

class ZmqReactor(ZmqBaseReactor):
    """
    A consumer class implementing a
    consumer for messages. Can also be
    used as a 1:1 proxy
    """

    def __init__(self, conf):
        super(ZmqReactor, self).__init__(conf)

    def consume(self, sock):
        #TODO(ewindisch): use zero-copy (i.e. references, not copying)
        data = sock.recv()
        LOG.debug(_("CONSUMER RECEIVED DATA: %s"), data)
        if sock in self.mapping:
            LOG.debug(_("ROUTER RELAY-OUT %(data)s") % {
                'data': data})
            self.mapping[sock].send(data)
            return

        proxy = self.proxies[sock]

        if data[2] == 'cast':  # Legacy protocol
            packenv = data[3]

            ctx, msg = _deserialize(packenv)
            request = rpc_common.deserialize_msg(msg)
            ctx = RpcContext.unmarshal(ctx)
        elif data[2] == 'impl_zmq_v2':
            packenv = data[4:]

            msg = unflatten_envelope(packenv)
            request = rpc_common.deserialize_msg(msg)

            # Unmarshal only after verifying the message.
            ctx = RpcContext.unmarshal(data[3])
        else:
            LOG.error(_("ZMQ Envelope version unsupported or unknown."))
            return

        self.pool.spawn_n(self.process, proxy, ctx, request)

class Connection(rpc_common.Connection):
    """Manages connections and threads."""

    def __init__(self, conf):
        self.topics = []
        self.reactor = ZmqReactor(conf)

    def create_consumer(self, topic, proxy, fanout=False):
        # Register with matchmaker.
        _get_matchmaker().register(topic, CONF.rpc_zmq_host)

        # Subscription scenarios
        if fanout:
            sock_type = zmq.SUB
            subscribe = ('', fanout)[type(fanout) == str]
            topic = 'fanout~' + topic.split('.', 1)[0]
        else:
            sock_type = zmq.PULL
            subscribe = None
            topic = '.'.join((topic.split('.', 1)[0], CONF.rpc_zmq_host))

        if topic in self.topics:
            LOG.info(_("Skipping topic registration. Already registered."))
            return

        # Receive messages from (local) proxy
        inaddr = "ipc://%s/zmq_topic_%s" % \
            (CONF.rpc_zmq_ipc_dir, topic)

        LOG.debug(_("Consumer is a zmq.%s"),
                  ['PULL', 'SUB'][sock_type == zmq.SUB])

        self.reactor.register(proxy, inaddr, sock_type,
                              subscribe=subscribe, in_bind=False)
        self.topics.append(topic)

    def close(self):
        _get_matchmaker().stop_heartbeat()
        for topic in self.topics:
            _get_matchmaker().unregister(topic, CONF.rpc_zmq_host)

        self.reactor.close()
        self.topics = []

    def wait(self):
        self.reactor.wait()

    def consume_in_thread(self):
        _get_matchmaker().start_heartbeat()
        self.reactor.consume_in_thread()

def _cast(addr, context, topic, msg, timeout=None, envelope=False,
          _msg_id=None):
    timeout_cast = timeout or CONF.rpc_cast_timeout
    payload = [RpcContext.marshal(context), msg]

    with Timeout(timeout_cast, exception=rpc_common.Timeout):
        try:
            conn = ZmqClient(addr)

            # assumes cast can't return an exception
            conn.cast(_msg_id, topic, payload, envelope)
        except zmq.ZMQError:
            raise RPCException("Cast failed. ZMQ Socket Exception")
        finally:
            if 'conn' in vars():
                conn.close()

def _call(addr, context, topic, msg, timeout=None,
          envelope=False):
    # timeout_response is how long we wait for a response
    timeout = timeout or CONF.rpc_response_timeout

    # The msg_id is used to track replies.
    msg_id = uuid.uuid4().hex

    # Replies always come into the reply service.
    reply_topic = "zmq_replies.%s" % CONF.rpc_zmq_host

    LOG.debug(_("Creating payload"))
    # Curry the original request into a reply method.
    mcontext = RpcContext.marshal(context)
    payload = {
        'method': '-reply',
        'args': {
            'msg_id': msg_id,
            'context': mcontext,
            'topic': reply_topic,
            'msg': [mcontext, msg]
        }
    }

    LOG.debug(_("Creating queue socket for reply waiter"))

    # Messages arriving async.
    # TODO(ewindisch): have reply consumer with dynamic subscription mgmt
    with Timeout(timeout, exception=rpc_common.Timeout):
        try:
            msg_waiter = ZmqSocket(
                "ipc://%s/zmq_topic_zmq_replies.%s" %
                (CONF.rpc_zmq_ipc_dir,
                 CONF.rpc_zmq_host),
                zmq.SUB, subscribe=msg_id, bind=False
            )

            LOG.debug(_("Sending cast"))
            _cast(addr, context, topic, payload, envelope)

            LOG.debug(_("Cast sent; Waiting reply"))
            # Blocks until receives reply
            msg = msg_waiter.recv()
            LOG.debug(_("Received message: %s"), msg)
            LOG.debug(_("Unpacking response"))

            if msg[2] == 'cast':  # Legacy version
                raw_msg = _deserialize(msg[-1])[-1]
            elif msg[2] == 'impl_zmq_v2':
                rpc_envelope = unflatten_envelope(msg[4:])
                raw_msg = rpc_common.deserialize_msg(rpc_envelope)
            else:
                raise rpc_common.UnsupportedRpcEnvelopeVersion(
                    _("Unsupported or unknown ZMQ envelope returned."))

            responses = raw_msg['args']['response']
        # ZMQError trumps the Timeout error.
        except zmq.ZMQError:
            raise RPCException("ZMQ Socket Error")
        except (IndexError, KeyError):
            raise RPCException(_("RPC Message Invalid."))
        finally:
            if 'msg_waiter' in vars():
                msg_waiter.close()

    # It seems we don't need to do all of the following,
    # but perhaps it would be useful for multicall?
    # One effect of this is that we're checking all
    # responses for Exceptions.
    for resp in responses:
        if isinstance(resp, types.DictType) and 'exc' in resp:
            raise rpc_common.deserialize_remote_exception(CONF, resp['exc'])

    return responses[-1]

def _multi_send(method, context, topic, msg, timeout=None,
                envelope=False, _msg_id=None):
    """
    Wraps the sending of messages,
    dispatches to the matchmaker and sends
    message to all relevant hosts.
    """
    conf = CONF
    LOG.debug(_("%(msg)s") % {'msg': ' '.join(map(pformat, (topic, msg)))})

    queues = _get_matchmaker().queues(topic)
    LOG.debug(_("Sending message(s) to: %s"), queues)

    # Don't stack if we have no matchmaker results
    if len(queues) == 0:
        LOG.warn(_("No matchmaker results. Not casting."))
        # While not strictly a timeout, callers know how to handle
        # this exception and a timeout isn't too big a lie.
        raise rpc_common.Timeout(_("No match from matchmaker."))

    # This supports brokerless fanout (addresses > 1)
    for queue in queues:
        (_topic, ip_addr) = queue
        _addr = "tcp://%s:%s" % (ip_addr, conf.rpc_zmq_port)

        if method.__name__ == '_cast':
            eventlet.spawn_n(method, _addr, context,
                             _topic, msg, timeout, envelope,
                             _msg_id)
            return
        return method(_addr, context, _topic, msg, timeout,
                      envelope)

def create_connection(conf, new=True):
    return Connection(conf)

def multicall(conf, *args, **kwargs):
    """Multiple calls."""
    return _multi_send(_call, *args, **kwargs)

def call(conf, *args, **kwargs):
    """Send a message, expect a response."""
    data = _multi_send(_call, *args, **kwargs)
    return data[-1]

def cast(conf, *args, **kwargs):
    """Send a message expecting no reply."""
    _multi_send(_cast, *args, **kwargs)

def fanout_cast(conf, context, topic, msg, **kwargs):
    """Send a message to all listening and expect no reply."""
    # NOTE(ewindisch): fanout~ is used because it avoid splitting on .
    # and acts as a non-subtle hint to the matchmaker and ZmqProxy.
    _multi_send(_cast, context, 'fanout~' + str(topic), msg, **kwargs)

def notify(conf, context, topic, msg, envelope):
    """
    Send notification event.
    Notifications are sent to topic-priority.
    This differs from the AMQP drivers which send to topic.priority.
    """
    # NOTE(ewindisch): dot-priority in rpc notifier does not
    # work with our assumptions.
    topic = topic.replace('.', '-')
    cast(conf, context, topic, msg, envelope=envelope)

def cleanup():
    """Clean up resources in use by implementation."""
    global ZMQ_CTX
    if ZMQ_CTX:
        ZMQ_CTX.term()
    ZMQ_CTX = None

    global matchmaker
    matchmaker = None

def _get_ctxt():
    if not zmq:
        raise ImportError("Failed to import eventlet.green.zmq")

    global ZMQ_CTX
    if not ZMQ_CTX:
        ZMQ_CTX = zmq.Context(CONF.rpc_zmq_contexts)
    return ZMQ_CTX

def _get_matchmaker(*args, **kwargs):
    global matchmaker
    if not matchmaker:
        matchmaker = importutils.import_object(
            CONF.rpc_zmq_matchmaker, *args, **kwargs)
    return matchmaker



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "placement_groups"

    def basic(self, request, placement_group):
        LOG.info("placement_groups api view %s " % placement_group)
        return {
            "placement_group": {
                "id": placement_group['id'],
                "pg_id": placement_group['pgid'],
                "state": placement_group['state'],
                "up": placement_group['up'],
                "acting": placement_group['acting'],
            }
        }

    def _detail(self, request, placement_group):
        LOG.info("placement_groups api detail view %s " % placement_group)
        return {
            "placement_group": {
                "id": placement_group['id'],
                "pg_id": placement_group['pgid'],
                "state": placement_group['state'],
                "up": placement_group['up'],
                "acting": placement_group['acting'],
            }
        }

    def detail(self, request, placement_groups):
        return self._list_view(self._detail, request, placement_groups)

    def index(self, request, placement_groups):
        """Show a list of placement_groups without many details."""
        return self._list_view(self.basic, request, placement_groups)

    def _list_view(self, func, request, placement_groups):
        """Provide a view for a list of placement_groups."""
        placement_group_list = [func(request, placement_group)["placement_group"] for placement_group in placement_groups]
        placement_groups_dict = dict(placement_groups=placement_group_list)
        return placement_groups_dict




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from vsm import utils
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm import db
from vsm.openstack.common import log as logging
from vsm.api.views import agents as agents_views
from vsm import context
from vsm.agent import appnodes
from vsm.manifest.parser import ManifestParser
from vsm.openstack.common.db import exception as db_exc

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

def make_agent(elem, detailed=False):
    elem.set('id')
    elem.set('host')
    elem.set('service_id')
    elem.set('primary_public_ip')
    elem.set('secondary_public_ip')
    elem.set('cluster_ip')
    elem.set('raw_ip')
    elem.set('zone_id')
    elem.set('type')
    elem.set('cluster_id')
    elem.set('data_drives_number')
    elem.set('status')
    elem.set('id_rsa_pub')

    if detailed:
        pass

    #xmlutil.make_links(elem, 'links')

def _translate_agent_summary_view(agent):
    """Get detail of agent information."""
    d = {"id": agent.get("id", ""),
        "host": agent.get("host", ""),
        "primary_public_ip": agent.get("primary_public_ip", ""),
        "secondary_public_ip": agent.get("secondary_public_ip", ""),
        "cluster_ip": agent.get("cluster_ip", ""),
        "raw_ip": agent.get("raw_ip", ""),
        "service_id": agent.get("service_id", ""),
        "zone_id": agent.get("zone_id", ""),
        "cluster_id": agent.get("cluster_id", ""),
        "data_drives_number": agent.get('data_drives_number', ""),
        "type": agent.get('type', ""),
        "id_rsa_pub": agent.get('id_rsa_pub', ""),
        "status": agent.get('status', "")}
    return d

agent_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class AgentTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('agent', selector='agent')
        make_agent(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=agent_nsmap)

class AgentsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('agents')
        elem = xmlutil.SubTemplateElement(root, 'agent', selector='agents')
        make_agent(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=agent_nsmap)

class AgentsController(wsgi.Controller):
    """The Agents API controller for the OpenStack API.

    -----------------
    NOTE: Do not use rpcapi class in Controller class.
    Try to avoid using RPC in services init function.

    Because after the services is running. RabbitMQ can
    be using for it.

    ==
    So just use db here.
    """
    _view_builder_class = agents_views.ViewBuilder

    def _get_keyring_admin_from_db(self):
        """Get keyring from DB."""

        # If already have keyring.admin just fetch once.
        if self._cluster_info.get('keyring_admin', None):
            LOG.info(' API contains the keyring.admin')
            return

        cluster_info = self._cluster_info['cluster']
        cluster_name = cluster_info['cluster_name']
        cluster_ref = db.cluster_get_by_name(self._context,
                                             cluster_name)

        if not cluster_ref:
            LOG.info(' No keyring_admin cluster = %s' % cluster_name)
            return
        else:
            # NOTE we update the cluster info in DB.
            LOG.info(' Find cluster name = %s' % cluster_name)
            # If we find the cluster, we also return the keyring.admin
            # info for storage nodes.
            info_dict = cluster_ref.get('info_dict', None)
            if info_dict:
                keyring_admin = json.loads(info_dict)
                keyring_admin_info = keyring_admin.get('keyring_admin', None)
                if keyring_admin:
                    self._cluster_info['keyring_admin'] = keyring_admin_info
                    LOG.info(' API get keyring.admin from DB.')
                else:
                    LOG.info('Can not get keyring_admin from DB.')
            else:
                LOG.info('Can not get info_dict from DB.')
            return True

    def _write_cluster_info(self):
        """Write cluster info into DB.

        Info includes:

            cluster_name
            file_system
            public_address
            secondary_public_address
            cluster_address
        """
        cluster_info = self._cluster_info['cluster']

        LOG.info(' cluster_info to db = %s' % \
            json.dumps(cluster_info, sort_keys=True, indent=4))

        cluster_name = cluster_info['cluster_name']

        cluster_ref = db.cluster_get_by_name(self._context,
                                             cluster_name)

        if not cluster_ref:
            LOG.info(' Have not find cluster = %s' % cluster_name)
            LOG.info(' Before Writing cluster = %s' % \
            json.dumps(cluster_info, sort_keys=True, indent=4))
            db.cluster_create(self._context, cluster_info)
        else:
            # NOTE we update the cluster info in DB.
            LOG.info(' Find cluster name = %s' % cluster_name)
            db.cluster_update(self._context,
                              cluster_ref['id'],
                              cluster_info)
            # If we find the cluster, we also return the keyring.admin
            # info for storage nodes.
            info_dict = cluster_ref['info_dict']
            if info_dict:
                LOG.info('Get info dict from DB.')
                keyring_admin = json.loads(info_dict).get('keyring_admin', None)
                self._cluster_info['keyring_admin'] = keyring_admin
            else:
                LOG.info('Can not get keyring from DB.')
            return True

    def _write_zone_info(self):
        """Write zone into DB.

        Zone list:
            zone_a
            zone_b
            zone_c

        Just write names here.
        """
        #TODO add zone table two variables. cluster_id, storage_group_id.
        zone_list = self._cluster_info['zone']
        for zone_name in zone_list:
            zone_ref = db.\
                          zone_get_by_name(self._context, zone_name)
            if not zone_ref:
                LOG.info('Have not find zone = %s' % zone_name)
                db.zone_create(self._context,
                                               {'name': zone_name,
                                                'deleted': False})
            else:
                LOG.info('Find zone = %s in zone Table.' % zone_name)
        return True

    def _write_ec_profiles(self):
        if not self._cluster_info.get('ec_profiles', None):
            return True

        profile_list = self._cluster_info['ec_profiles']
        for profile in profile_list:
            db.ec_profile_update_or_create(self._context, profile) 

        return True

    def _write_cache_tier_defaults(self):
        if not self._cluster_info.get('cache_tier_defaults', None):
            return True
        cache_tier_defaults = self._cluster_info['cache_tier_defaults']
        LOG.info("CLUSTER INFO")
        LOG.info(cache_tier_defaults)
        name_list = []
        sets = db.vsm_settings_get_all(self._context)
        db_list = [s.name for s in sets]
        for setting in cache_tier_defaults:
            LOG.info(setting)
            name = setting.get('name', None)
            value = setting.get('default_value', None)
            if not name in db_list:
                try:
                    if not value.isdigit():
                        value = float(value)
                except ValueError:
                    value = None
                except AttributeError:
                    value = None
                if not value:
                    LOG.warn('The default value of %s should be digit. Load default value ...' % name)
                    value = FLAGS.get(name, None)
                    if not value:
                        LOG.warn('Failed to load the default value of %s.' % name)
                        try:
                            raise exception.GetNoneError
                        except exception.GetNoneError as e:
                            LOG.error("%s:%s" %(e.code, e.message))
                        ret = False
                        continue
                setting['value'] = value
                ref = db.vsm_settings_update_or_create(self._context, setting)
                name_list.append(name)




    def _write_vsm_settings(self):
        """Writing vsm settings into DB.

           [settings]
           #format [key] [default_value]
           storage_group_near_full_threshold 65
           storage_group_full_threshold 85

        """
        ret = True
        setting_list = self._cluster_info['settings']
        name_list = list()
        sets = db.vsm_settings_get_all(self._context)
        db_list = [s.name for s in sets]
        LOG.debug('vsm settings already exists in db: %s' % db_list)

        for setting in setting_list:
            try:
                name = setting.get('name', None)
                value = setting.get('default_value', None)
                # The setting does not exist in db.
                if not name in db_list:
                    if not value or not value.isdigit():
                        LOG.warn('The default value of %s should be digit. Load default value ...' % name)
                        value = FLAGS.get(name, None)
                        if not value:
                            LOG.warn('Failed to load the default value of %s.' % name)
                            try:
                                raise exception.GetNoneError
                            except exception.GetNoneError as e:
                                LOG.error("%s:%s" %(e.code, e.message))
                            ret = False
                            continue
                    setting['value'] = value
                    ref = db.vsm_settings_update_or_create(self._context,
                                                           setting)
                    name_list.append(name)
            except:
                ret = False
                LOG.error('Failed to load setting %s.' % name)
                continue

        # load other default vsm settings from flags
        setting_list = name_list + db_list
        LOG.debug('settings loaded: %s' % setting_list)
        for ss in flags.vsm_settings_opts:

            if not ss.name in setting_list:
                try:
                    val = {
                        'name': ss.name,
                        'value': str(ss.default),
                        'default_value': str(ss.default)
                    }
                    ref = db.vsm_settings_update_or_create(self._context,
                                                           val)
                except:
                    ret = False
                    LOG.error('Failed to load setting %s from flags.' % name)
                continue

        return ret

    def _write_openstack_ip(self):
        """ Write Openstack nova controller ips
            and cinder volume ips into DB.

         Openstack Ips:
            10.239.82.xx
            10.239.82.yy
         Just write Ip addr here.
        """
        ip_list = self._cluster_info['openstack_ip']
        if ip_list:
            try:
                # fake one user id and project id
                self._context.user_id = 32 * '1'
                self._context.project_id = 32 * '1'
                node_list = appnodes.create(self._context, ip_list, allow_duplicate=True)
                #LOG.debug('app nodes added %s' % node_list)
                for node in node_list:
                    status = 'reachable'
                    appnodes.update(self._context, node.id, status)
            except Exception as e:
                LOG.error('Failed to add app nodes with error %s' % e)
                return False
        return True

    def _write_storage_groups(self):
        """Write storage groups into DB.

        At here we have to write info as below:

            name, storage_class, friendly_name, rule_id, deleted,take_order.

        --------------------------------------------------------
        Important
        =========
        rule_id, in ceph system. The must be started from 0.
        Assume there are 0, 1, 2, 3 four storage groups, delete the
        iter=2 storage group, then all numbers will changed to be
        0 1 2. In fact, old {3}'s number is changed to be 2.

        For insert, we must follow one by one.
        """
        #TODO storage group need to contains cluster_id.

        stg_list = self._cluster_info['storage_group']
        for rule_id, stg in enumerate(stg_list):
            stg['rule_id'] = rule_id
            stg['status'] = FLAGS.storage_group_in
            stg['take_order'] = 0
            stg['choose_num'] = 0
            stg['choose_type'] = 'host'
            # TODO change the create_storage_group api.
            # DB create_storage_group will ignore the
            # request if the stg has exists.
            db.create_storage_group(self._context,
                        values=stg)
            LOG.info('Write storage group = %s success.' % stg['name'])
        return True

    def _store_cluster_info_to_db(self):
        """Store the cluster.manifest info into db.

        We have to insert info into db as follow:

            1. cluster name, public addresses etc.
            2. zone list.
            3. storage group list.

        Although there four segs in cluster.manifest.
        [stoarge_class] and [storage_group] refer to
        storage_group.

        """
        # TODO add info into db.tables.
        # Need Zone tables containse cluster.id ad forein key.
        # Insert cluster name and addresses.
        LOG.info(' agents.py _store_cluster_info_to_db()')
        write_cluster_ret = self._write_cluster_info()
        write_zone_ret = self._write_zone_info()
        write_storage_groups = self._write_storage_groups()
        write_openstack_ip = self._write_openstack_ip()
        write_settings = self._write_vsm_settings()
        write_ec_profiles = self._write_ec_profiles()
        write_cache_tier_defaults = self._write_cache_tier_defaults()

        self._have_write_cluter_into_db = \
            write_cluster_ret and write_zone_ret \
            and write_storage_groups and write_openstack_ip \
            and write_settings and write_ec_profiles \
            and write_cache_tier_defaults

    def __init__(self, ext_mgr):
        self.ext_mgr = ext_mgr
        self._context = context.get_admin_context()
        self.smp = ManifestParser(FLAGS.cluster_manifest, False)
        self._cluster_info = self.smp.format_to_json()
        LOG.info(' Get data from manifest file = %s' %\
            json.dumps(self._cluster_info, sort_keys=True, indent=4))
        vsm_conf = open(FLAGS.vsm_config, 'r').read()
        api_paste = open(FLAGS.api_paste_config, 'r').read()
        self._cluster_info['vsm.conf'] = vsm_conf
        self._cluster_info['api-paste.ini'] = api_paste
        self._have_write_cluter_into_db = False
        LOG.info(' running before writing DB.')
        self._store_cluster_info_to_db()
        super(AgentsController, self).__init__()

    def _get_agent_search_options(self):
        """Return agent search options allowed by non-admin."""
        return ('id', 'name', 'public_ip')

    def _get_storage_group_and_class_list(self):
        list = db.storage_group_get_all(self._context)
        storage_group_list = []
        storage_class_list = []
        if list:
            for item in list:
                value = {}
                value['friendly_name'] = item['friendly_name']
                value['name'] = item['name']
                value['rule_id'] = item['rule_id']
                value['storage_class'] = item['storage_class']
                storage_group_list.append(value)
                if not item['storage_class'] in storage_class_list:
                    storage_class_list.append(item['storage_class'])
        return storage_group_list, storage_class_list

    @wsgi.serializers(xml=AgentsTemplate)
    def index(self, req):
        """Return vsm-controller's info and configures."""
        try:
            self._get_keyring_admin_from_db()
            if self._have_write_cluter_into_db == False:
                self._store_cluster_info_to_db()
        except:
            LOG.error('Write DB error occurs in agents.index()')

        if req.environ['vsm.context']:
            LOG.info('context is not None')
        self._cluster_info['etc_hosts'] = \
             utils.read_file_as_root(FLAGS.etc_hosts)
        storage_group , storage_class = self._get_storage_group_and_class_list()
        self._cluster_info['storage_class'] = storage_class
        self._cluster_info['storage_group'] = storage_group
        return self._cluster_info

def create_resource(ext_mgr):
    return wsgi.Resource(AgentsController(ext_mgr))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 OpenStack, LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Common Policy Engine Implementation"""

import logging
import urllib
import urllib2

from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import jsonutils

LOG = logging.getLogger(__name__)

_BRAIN = None

def set_brain(brain):
    """Set the brain used by enforce().

    Defaults use Brain() if not set.

    """
    global _BRAIN
    _BRAIN = brain

def reset():
    """Clear the brain used by enforce()."""
    global _BRAIN
    _BRAIN = None

def enforce(match_list, target_dict, credentials_dict, exc=None,
            *args, **kwargs):
    """Enforces authorization of some rules against credentials.

    :param match_list: nested tuples of data to match against

        The basic brain supports three types of match lists:

            1) rules

                looks like: ``('rule:compute:get_instance',)``

                Retrieves the named rule from the rules dict and recursively
                checks against the contents of the rule.

            2) roles

                looks like: ``('role:compute:admin',)``

                Matches if the specified role is in credentials_dict['roles'].

            3) generic

                looks like: ``('tenant_id:%(tenant_id)s',)``

                Substitutes values from the target dict into the match using
                the % operator and matches them against the creds dict.

        Combining rules:

            The brain returns True if any of the outer tuple of rules
            match and also True if all of the inner tuples match. You
            can use this to perform simple boolean logic.  For
            example, the following rule would return True if the creds
            contain the role 'admin' OR the if the tenant_id matches
            the target dict AND the the creds contains the role
            'compute_sysadmin':

            ::

                {
                    "rule:combined": (
                        'role:admin',
                        ('tenant_id:%(tenant_id)s', 'role:compute_sysadmin')
                    )
                }

        Note that rule and role are reserved words in the credentials match, so
        you can't match against properties with those names. Custom brains may
        also add new reserved words. For example, the HttpBrain adds http as a
        reserved word.

    :param target_dict: dict of object properties

      Target dicts contain as much information as we can about the object being
      operated on.

    :param credentials_dict: dict of actor properties

      Credentials dicts contain as much information as we can about the user
      performing the action.

    :param exc: exception to raise

      Class of the exception to raise if the check fails.  Any remaining
      arguments passed to enforce() (both positional and keyword arguments)
      will be passed to the exception class.  If exc is not provided, returns
      False.

    :return: True if the policy allows the action
    :return: False if the policy does not allow the action and exc is not set
    """
    global _BRAIN
    if not _BRAIN:
        _BRAIN = Brain()
    if not _BRAIN.check(match_list, target_dict, credentials_dict):
        if exc:
            raise exc(*args, **kwargs)
        return False
    return True

class Brain(object):
    """Implements policy checking."""

    _checks = {}

    @classmethod
    def _register(cls, name, func):
        cls._checks[name] = func

    @classmethod
    def load_json(cls, data, default_rule=None):
        """Init a brain using json instead of a rules dictionary."""
        rules_dict = jsonutils.loads(data)
        return cls(rules=rules_dict, default_rule=default_rule)

    def __init__(self, rules=None, default_rule=None):
        if self.__class__ != Brain:
            LOG.warning(_("Inheritance-based rules are deprecated; use "
                          "the default brain instead of %s.") %
                        self.__class__.__name__)

        self.rules = rules or {}
        self.default_rule = default_rule

    def add_rule(self, key, match):
        self.rules[key] = match

    def _check(self, match, target_dict, cred_dict):
        try:
            match_kind, match_value = match.split(':', 1)
        except Exception:
            LOG.exception(_("Failed to understand rule %(match)r") % locals())
            # If the rule is invalid, fail closed
            return False

        func = None
        try:
            old_func = getattr(self, '_check_%s' % match_kind)
        except AttributeError:
            func = self._checks.get(match_kind, self._checks.get(None, None))
        else:
            LOG.warning(_("Inheritance-based rules are deprecated; update "
                          "_check_%s") % match_kind)
            func = lambda brain, kind, value, target, cred: old_func(value,
                                                                     target,
                                                                     cred)

        if not func:
            LOG.error(_("No handler for matches of kind %s") % match_kind)
            # Fail closed
            return False

        return func(self, match_kind, match_value, target_dict, cred_dict)

    def check(self, match_list, target_dict, cred_dict):
        """Checks authorization of some rules against credentials.

        Detailed description of the check with examples in policy.enforce().

        :param match_list: nested tuples of data to match against
        :param target_dict: dict of object properties
        :param credentials_dict: dict of actor properties

        :returns: True if the check passes

        """
        if not match_list:
            return True
        for and_list in match_list:
            if isinstance(and_list, basestring):
                and_list = (and_list,)
            if all([self._check(item, target_dict, cred_dict)
                    for item in and_list]):
                return True
        return False

class HttpBrain(Brain):
    """A brain that can check external urls for policy.

    Posts json blobs for target and credentials.

    Note that this brain is deprecated; the http check is registered
    by default.
    """

    pass

def register(name, func=None):
    """
    Register a function as a policy check.

    :param name: Gives the name of the check type, e.g., 'rule',
                 'role', etc.  If name is None, a default function
                 will be registered.
    :param func: If given, provides the function to register.  If not
                 given, returns a function taking one argument to
                 specify the function to register, allowing use as a
                 decorator.
    """

    # Perform the actual decoration by registering the function.
    # Returns the function for compliance with the decorator
    # interface.
    def decorator(func):
        # Register the function
        Brain._register(name, func)
        return func

    # If the function is given, do the registration
    if func:
        return decorator(func)

    return decorator

@register("rule")
def _check_rule(brain, match_kind, match, target_dict, cred_dict):
    """Recursively checks credentials based on the brains rules."""
    try:
        new_match_list = brain.rules[match]
    except KeyError:
        if brain.default_rule and match != brain.default_rule:
            new_match_list = ('rule:%s' % brain.default_rule,)
        else:
            return False

    return brain.check(new_match_list, target_dict, cred_dict)

@register("role")
def _check_role(brain, match_kind, match, target_dict, cred_dict):
    """Check that there is a matching role in the cred dict."""
    return match.lower() in [x.lower() for x in cred_dict['roles']]

@register('http')
def _check_http(brain, match_kind, match, target_dict, cred_dict):
    """Check http: rules by calling to a remote server.

    This example implementation simply verifies that the response is
    exactly 'True'. A custom brain using response codes could easily
    be implemented.

    """
    url = 'http:' + (match % target_dict)
    data = {'target': jsonutils.dumps(target_dict),
            'credentials': jsonutils.dumps(cred_dict)}
    post_data = urllib.urlencode(data)
    f = urllib2.urlopen(url, post_data)
    return f.read() == "True"

@register(None)
def _check_generic(brain, match_kind, match, target_dict, cred_dict):
    """Check an individual match.

    Matches look like:

        tenant:%(tenant_id)s
        role:compute:admin

    """

    # TODO(termie): do dict inspection via dot syntax
    match = match % target_dict
    if match_kind in cred_dict:
        return match == unicode(cred_dict[match_kind])
    return False



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "agents"

    def basic(self, agents):
        if not agents.get('host', ""):
            agents['host'] = ""
            return agents
        return {
            "agents": {
                "id": agents.get("id", ""),
                "host": agents.get("host", ""),
                "primary_public_ip": agents.get("primary_public_ip", ""),
                "secondary_public_ip": agents.get("secondary_public_ip", ""),
                "cluster_ip": agents.get("cluster_ip", ""),
                "cluster_id": agents.get("cluster_id", ""),
                "data_drives_number": agents.get('data_drives_number', ""),
                "id_rsa_pub": agents.get('id_rsa_pub', ""),
                "raw_ip": agents.get('raw_ip', ""),
                "zone_id": agents.get("zone_id", ""),
                "type": agents.get('type', ""),
                "status": agents.get('status', ""),
                "service_id": agents.get('service_id', "")
            }
        }

    def index(self, agentss):
        """Show a list of agentss without many details."""
        return self._list_view(self.basic, agentss)

    def _list_view(self, func, agentss):
        """Provide a view for a list of agentss."""
        agents_list = [func(agents)["agents"] for agents in agentss]
        agentss_dict = dict(agentss=agents_list)
        return agentss_dict



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import Boolean, Column, DateTime, ForeignKey
from sqlalchemy import Integer, MetaData, String, Table
from sqlalchemy import Table, Text, Float, Index
#test
def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine;
    # bind migrate_engine to your metadata
    meta = MetaData()
    meta.bind = migrate_engine

    metrics = Table('metrics', meta, autoload=True)
    metrics_timestamp_index = Index('metrics_timestamp_index',metrics.columns.timestamp)
    metrics_timestamp_index.create(bind=migrate_engine)


def downgrade(migrate_engine):
    pass



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010-2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from vsm.openstack.common import timeutils

class ViewBuilder(object):
    """OpenStack API base limits view builder."""

    def build(self, rate_limits, absolute_limits):
        rate_limits = self._build_rate_limits(rate_limits)
        absolute_limits = self._build_absolute_limits(absolute_limits)

        output = {
            "limits": {
                "rate": rate_limits,
                "absolute": absolute_limits,
            },
        }

        return output

    def _build_absolute_limits(self, absolute_limits):
        """Builder for absolute limits

        absolute_limits should be given as a dict of limits.
        For example: {"ram": 512, "gigabytes": 1024}.

        """
        limit_names = {
            "ram": ["maxTotalRAMSize"],
            "instances": ["maxTotalInstances"],
            "cores": ["maxTotalCores"],
            "gigabytes": ["maxTotalHardwareGigabytes"],
            "storages": ["maxTotalHardwares"],
            "key_pairs": ["maxTotalKeypairs"],
            "floating_ips": ["maxTotalFloatingIps"],
            "metadata_items": ["maxServerMeta", "maxImageMeta"],
            "injected_files": ["maxPersonality"],
            "injected_file_content_bytes": ["maxPersonalitySize"],
        }
        limits = {}
        for name, value in absolute_limits.iteritems():
            if name in limit_names and value is not None:
                for name in limit_names[name]:
                    limits[name] = value
        return limits

    def _build_rate_limits(self, rate_limits):
        limits = []
        for rate_limit in rate_limits:
            _rate_limit_key = None
            _rate_limit = self._build_rate_limit(rate_limit)

            # check for existing key
            for limit in limits:
                if (limit["uri"] == rate_limit["URI"] and
                    limit["regex"] == rate_limit["regex"]):
                    _rate_limit_key = limit
                    break

            # ensure we have a key if we didn't find one
            if not _rate_limit_key:
                _rate_limit_key = {
                    "uri": rate_limit["URI"],
                    "regex": rate_limit["regex"],
                    "limit": [],
                }
                limits.append(_rate_limit_key)

            _rate_limit_key["limit"].append(_rate_limit)

        return limits

    def _build_rate_limit(self, rate_limit):
        _get_utc = datetime.datetime.utcfromtimestamp
        next_avail = _get_utc(rate_limit["resetTime"])
        return {
            "verb": rate_limit["verb"],
            "value": rate_limit["value"],
            "remaining": int(rate_limit["remaining"]),
            "unit": rate_limit["unit"],
            "next-available": timeutils.isotime(at=next_avail),
        }



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Use socket to transfer msg.
"""

import json
import urllib2
from vsm import flags
from vsm.openstack.common import log as logging
from vsm import utils
import token

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

class WSGIClient(object):
    """WSGI Client service to get data from vsm-api.

    When used by vsm-agen, the password is get from server.manifest.
    """
    def __init__(self, vsm_api_ip='vsm_api_ip', info='token-tenantid'):
        """Initialized the url requestion and RUL."""
        self._vsm_api_ip = vsm_api_ip
        self._token = "-".join(info.split("-")[0:-1])
        self._tenant_id = info.split("-")[-1]
        self._vsm_url = "http://%s:%s/v1/%s" % \
                        (self._vsm_api_ip,
                         8778,
                         self._tenant_id)
        LOG.info('Agent token = %s, access url = %s' % \
                 (self._token, self._vsm_url))

    def index(self):
        """Use this method to get cluster's critial information."""

        req_url = self._vsm_url + "/agents"
        req = urllib2.Request(req_url)
        req.get_method = lambda: 'GET'
        req.add_header("content-type", "application/json")
        req.add_header("X-Auth-Token", self._token)
        resp = urllib2.urlopen(req)
        recive_data = json.loads(resp.read())
        return recive_data

    def send(self, data, method, url="agents"):
        """Use parameter to contruct wsgi request to server.

        Below is an example how to send wsgi request to vsm-api server.

            url = "http://%(host)s:%(port)s/gw/%(id)/agents"
            req = urllib2.Request(url,
                  data=json.dumps({"agent":{"name":"test"}}))
            req.get_method = lambda: "POST"
            req.add_header("content-type", "application/json")
            resp = urllib2.urlopen(req)
            configs = json.loads(resp.read())

        It's easy to transfer %(id), for simple, you can juse transfer 1.
        """
        pass



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import and_, String, Column, MetaData, select, Table, Integer

def upgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    storage_pools = Table('storage_pools', meta, autoload=True)
    primary_storage_group_id = Column('primary_storage_group_id', Integer)

    storage_pools.create_column(primary_storage_group_id)

    storage_groups = Table('storage_groups', meta, autoload=True)

    #set storage_pools.primary_stoarge_group_id = storage_group.id
    # set instances.node = compute_nodes.hypervisore_hostname
    q = select(
            [storage_pools.c.id, storage_groups.c.id],
            whereclause=and_(
                storage_pools.c.deleted != True,
                storage_groups.c.deleted != True),
            from_obj=storage_pools.join(storage_groups,
                     storage_pools.c.crush_ruleset == storage_groups.c.rule_id))
    for (storage_pool_id, storage_group_id) in q.execute():
        storage_pools.update().where(storage_pools.c.id == storage_pool_id).\
                                values(primary_storage_group_id=storage_group_id).\
                                execute()  

def downgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    storage_pools = Table('storage_pools', meta, autoload=True)

    storage_pools.drop_column('primary_storage_group_id')



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging
import time

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "monitors"

    def basic(self, mon):
        LOG.info("mon api view %s " % mon)
        return {
            "monitor": {
                "id": mon['id'],
                "name": mon['name'],
                "address": mon.get('address'),
                "health": mon.get('health'),
                "details": mon.get('details'),
            }
        }

    def show(self, mon):
        LOG.info("mon api view %s " % mon)
        _mon = {
            "monitor": {
                "id": mon['id'],
                "name": mon['name'],
                "address": mon.get('address'),
                "health": mon.get('health'),
                "details": mon.get('details'),
                "skew": float(mon.get('skew')),
                "latency": float(mon.get('latency')),
                "kb_total": mon.get('kb_total'),
                "kb_used": mon.get('kb_used'),
                "kb_avail": mon.get('kb_avail'),
                "avail_percent": mon.get('avail_percent')
            }
        }

        try:
            _mon['monitor']['updated_at'] = mon.get('updated_at').strftime("%Y-%m-%d %H:%M:%S")
        except:
            _mon['monitor']['updated_at'] = ""

        return _mon

    def index(self, mons):
        """Show a list of mons without many details."""
        return self._list_view(self.basic, mons)

    def detail(self, mons):
        return self._list_view(self.show, mons)

    def _list_view(self, func, mons):
        """Provide a view for a list of mons."""
        mon_list = [func(mon)["monitor"] for mon in mons]
        mons_dict = dict(monitors=mon_list)
        return mons_dict




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Crush map parser..

Parse crush map in json format, and identify storage groups from ruleset.

"""
import json
import operator

class CrushMap():

    def __init__(self, json_file=None, json_context=None):
        if json_file:
            with open(json_file, 'r') as fh:
                crush_map = json.load(fh)
            pass
        elif json_context:
            crush_map = json.loads(json_context)
        self._tunables = crush_map['tunables']
        self._devices = crush_map['devices']
        self._rules = crush_map['rules']
        self._types = crush_map['types']
        self._buckets = crush_map['buckets']


    def get_all_tunables(self):
        return self._tunables

    def get_tunable_value(self, parameter):
        return self._tunables[parameter]

    def get_all_types(self):
        return self._types

    def get_type_by_name(self, typename):
        return filter(lambda item: typename == item['name'], self._types)[0]

    def get_all_buckets(self):
        return self._buckets

    def get_weight_by_osd_name(self,osd_name):
        osds = self._devices
        osd_id = [osd['id'] for osd in osds if osd['name'] == osd_name]
        buckets = self._buckets
        for bucket in buckets:
            for item in bucket['items']:
                if item['id'] == osd_id:
                    return item['weight']
        return 'no osd:%s'%osd_name

    def get_bucket_by_id(self, id):
        return filter(lambda item: id == item['id'], self._buckets)[0]

    def get_buckets_by_type(self, typename):
        type = self.get_type_by_name(typename)
        if type:
            type_id = type['type_id']
            return filter(lambda item: type_id == item['type_id'], self._buckets)
        else:
            return []

    def get_buckets_by_name(self, name):
        for bucket in self._buckets:
            if name == bucket['name']:
                return bucket

    def get_children_by_type(self, id, type):
        children = []
        bucket = self.get_bucket_by_id(id)

        for item in bucket['items']:
            child = self.get_bucket_by_id(item['id'])
            if type == child['type_name']:
                children.append(child)

        return children

    def get_all_rules(self):
        return self._rules

    def get_rules_by_name(self, name):
        for rule in self._rules:
            if name == rule['rule_name']:
                return rule
        return None

    def get_rules_by_id(self, rule_id):
        for rule in self._rules:
            if rule_id == rule['rule_id']:
                return rule

    def get_osd_by_id(self, id):
        osd = filter(lambda item: id == item['id'], self._devices)
        if osd:
            return osd[0]

    def get_all_osds_by_rule(self, name):
        rule = self.get_rules_by_name(name)
        steps = rule and rule['steps'] or []
        devices = []
        for step in steps:
            if step['op'] == 'take':
                bucket_id = step['item']
                self.get_all_osds_by_bucket(bucket_id, devices)
        return devices

    def osd_count_by_rule_id(self,rule_id):
        rule = self.get_rules_by_id(rule_id)
        steps = rule['steps']
        devices = []
        for step in steps:
            if step['op'] == 'take':
                bucket_id = step['item']
                self.get_all_osds_by_bucket(bucket_id, devices)
        return len(devices)


    def get_all_osds_by_bucket(self, id, devices):
        print id
        if id >=0 :
            osd = self.get_osd_by_id(id)
            devices.append(osd)
        else :
            bucket = self.get_bucket_by_id(id)
            if bucket :
                items = bucket['items']
                for item in items:
                    self.get_all_osds_by_bucket(item['id'], devices)

        return devices

    def get_bucket_root_by_rule_name(self, rule_name):
        '''
        this function will try to get bucket root for ec pool from storage group
        :param rule:
        :return:
        '''
        rule = self.get_rules_by_name(rule_name)
        bucket_root = "default"

        if len(rule['steps']) > 0:
            step = rule['steps'][0]
            bucket_root = step['item_name']

        return bucket_root

    def get_storage_groups_by_rule(self, rule):
        '''
        this function will execute each op.in detail,
            - op: take  --> the start of an new search
            - op: chooseleaf_firstn/... --> the rule to search
            - op: emit --> the end of a search
        '''
        storage_groups = []
        buckets = []
        sg_count = 0

        for step in rule['steps']:
            op = step['op']
            if op == 'take':
                sg_count += 1
                storage_groups.append([])
                take_bucket = self.get_bucket_by_id(step['item'])
                if take_bucket:
                    buckets.append(take_bucket)
                else:
                    return "undefined error:item %s"%(str(step['item']))

            if op in ['choose_firstn', 'chooseleaf_firstn', 'choose_indep', 'chooseleaf_indep']:
                type = step['type']
                children = []
                for bucket in buckets:
                    temp_buckets = self.get_children_by_type(bucket['id'], type)
                    for temp_bucket in temp_buckets:
                        children.append(temp_bucket)

                buckets = children

            if op == 'emit':
                if sg_count <= 0 :
                    return "invalid crush map format, take and emit is not in pair"
                for bucket in buckets:
                    devices = []
                    storage_groups[sg_count-1] = self.get_all_osds_by_bucket(bucket['id'],devices)

        print storage_groups
        return storage_groups

    def get_storage_group_value_by_rule_name(self,rule_name):
        rule = self.get_rules_by_name(rule_name)
        values = self.get_storage_groups_dict_by_rule([rule])
        return values

    def get_storage_groups_dict_by_rule(self, rules):
        '''
        this function will execute each op.in detail,
            - op: take  --> the start of an new search
            - op: chooseleaf_firstn/... --> the rule to search
            - op: emit --> the end of a search
        '''
        storage_groups = []
        for rule in rules:
            index = -1
            steps_len = len(rule['steps'])
            for i in range(steps_len):
                step = rule['steps'][i]
                op = step['op']
                if op == 'take':
                    bucket_id =  step['item']
                    index = index + 1
                    next_step = rule['steps'][i+1]
                    values = {'name': rule['rule_name'],
                              'choose_num': next_step.get('num',None) or 0,
                              'choose_type': next_step.get('type',None) or '',
                              'take_id': bucket_id,
                              'rule_id': rule['rule_id'],
                              'take_order': index,
                              }
                    storage_groups.append(values)

        return storage_groups

    def _get_location_by_osd_name_list(self,osd_name_list):
        parent_bucket = []
        osds = self._devices
        buckets = self._buckets
        for osd_name in osd_name_list:
            osd_id = [osd['id'] for osd in osds if osd['name'] == osd_name]
            for bucket in buckets:
                for item in bucket['items']:
                    if item['id'] == osd_id:
                        parent_bucket.append(bucket['id'])
        return list(set(parent_bucket))


    def _get_location_by_osd_name(self,osd_name):
        parent_bucket = {}
        osds = self._devices
        buckets = self._buckets
        osd_id = [osd['id'] for osd in osds if osd['name'] == osd_name]
        osd_id = osd_id[0]
        for bucket in buckets:
            for item in bucket['items']:
                if int(item['id']) == int(osd_id):
                    parent_bucket['name'] = bucket['name']
                    parent_bucket['type_name'] = bucket['type_name']
                    break
        return parent_bucket

    def get_parent_bucket_by_name(self, name):
        parent_bucket = {}
        self_bucket = self.get_buckets_by_name(name)
        if self_bucket:
            bucket_id = self_bucket['id']
            buckets = self._buckets
            for bucket in buckets:
                for item in bucket['items']:
                    if int(bucket_id) == int(item['id']):
                        parent_bucket['id'] = bucket['id']
                        parent_bucket['name'] = bucket['name']
                        parent_bucket['type_name'] = bucket['type_name']
                        break
        return parent_bucket

    def get_zone_id_by_host_name(self, host_name):
        zone = self.get_parent_bucket_by_name(host_name)
        return zone['id'] if zone else None

    def get_zone_id_by_osd_name(self, osd_name):
        host_bucket = self._get_location_by_osd_name(osd_name)
        host_name = host_bucket['name']
        return self.get_zone_id_by_host_name(host_name)

    def _show_as_tree_dict(self):
        buckets = self._buckets
        tree_data = {}
        types = self._types
        types.sort(key=operator.itemgetter('type_id'))
        for bucket in buckets:
            node_id = bucket['id']
            items = bucket['items']
            if not tree_data.has_key(str(node_id)):
                node_name = bucket['name']
                node_type_id = bucket['type_id']
                tree_node_data = {'id':node_id,'name':node_name,'type':node_type_id,'type_name':bucket['type_name']}
                tree_data[str(node_id)] = tree_node_data
            for item in items:
                item_id = item['id']
                if self.get_osd_by_id(item_id):
                    item_node = self.get_osd_by_id(item_id)
                    item_node['type_id'] = types[0]['type_id']
                    item_node['type_name'] = types[0]['name']
                elif self.get_bucket_by_id(item_id):
                    item_node = self.get_bucket_by_id(item_id)
                else:
                    return 'No defined error in crushmap:id=%s'%item_id
                item_node_id = item_node['id']
                item_node_parent_id = [node_id]
                if tree_data.has_key(str(item_node_id)):
                    tree_data[str(item_node_id)]['parent_id'] = tree_data[str(item_node_id)].get('parent_id',[])+item_node_parent_id
                else:
                    item_node_name = item_node['name']
                    item_node_type_id = item_node['type_id']
                    item_node_type_name = item_node['type_name']
                    item_tree_node_data = {'id':item_node_id,'name':item_node_name,'type':item_node_type_id,'type_name':item_node_type_name,'parent_id':item_node_parent_id}
                    tree_data[str(item_node_id)] = item_tree_node_data

        return tree_data.values()






if __name__ == '__main__':
    crushmap = CrushMap("./crush.json")
    # print '-----tree_data------'
    # crushmap._show_as_tree_dict()
    # print '=========tree_data================'
    # tunables = crushmap.get_all_tunables()
    ret = crushmap._get_location_by_osd_name('osd.1')
    print 'location----%s-'%ret

    bucket_root = crushmap.get_bucket_root_by_rule_id('performance')
    print 'bucket root = %s' %bucket_root

    ret = crushmap.get_storage_group_value_by_rule_name('ecpooltest3')
    print 'ecpooltest3---storage_group===',ret

#    for name in tunables:
#        print name, tunables[name]
#    print crushmap.get_tunable_value('profile')
#    buckets = crushmap.get_all_buckets()
#    bucket = crushmap.get_buckets_by_name('ceph01_high_performance_test_zone0')
#    print bucket

#    print crushmap.get_all_types()
#    print crushmap.get_all_buckets()
#    buckets = crushmap.get_buckets_by_type('zone')
#    print type(buckets)

#    print crushmap.get_rules_by_name('capacity')
#    crushmap.get_all_osds_by_rule('capacity')
#    print crushmap.get_bucket_by_id(-15)

#    print crushmap.get_rules_by_name('performance')

#    print crushmap.get_all_osds_by_bucket(-15, [])
#    bucket = crushmap.get_bucket_by_id(-15)
#    print bucket
#    print crushmap.get_children_by_type(-15, 'zone')
#     rule = crushmap.get_rules_by_name('value')
#     crushmap.get_storage_groups_by_rule(rule)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Crush map parser..

Parse crush map in json format, and identify storage groups from ruleset.

"""
import json
import operator
from vsm import utils
from vsm import exception
from vsm.openstack.common import log as logging
from vsm import flags

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

class cephconf_discover():

    def __init__(self, json_file=None, keyring=None):
        self._keyring = keyring
        if json_file:
            with open(json_file, 'r') as fh:
                ceph_report = json.load(fh)
            pass
        else:
            ceph_report = self._get_report()

        self._osds = {}
        self._mons = {}
        self.ceph_report = ceph_report
        if ceph_report:
            self._osds = ceph_report['osd_metadata']
            self._mons = ceph_report['monmap']['mons']


    def _get_report(self):
        report = ""
        stderr = ""
        try:
            if self._keyring:
                (report, stderr) = utils.execute('ceph', 'report','--keyring',self._keyring, run_as_root=True)
            else:
                (report, stderr) = utils.execute('ceph', 'report', run_as_root=True)
            return json.loads(report)
        except exception.ProcessExecutionError:
            LOG.warn('Fail to get report from ceph: %s' % stderr)


    def get_osds(self):
        return self._osds

    def get_osd(self, id=0):
        # osd_fields = ['devs','host','cluster addr','public addr','osd journal']
        if (id >= 0):
            if id < len(self._osds):
                return self._osds[id]

    def fixup_osd_settings(self):
        # enumerate osds
        osd_settings = "\n"
        for osd in self.get_osds():
            osd_settings += '[osd.%s]\n' %osd['id']
            osd_settings += 'host = %s\n' %osd['hostname']
            osd_settings += 'devs = %s\n' %osd['osd_data']
            osd_settings += 'osd journal = %s\n' %osd['osd_journal']
            osd_settings += 'cluster addr = %s\n' %osd['back_addr']
            osd_settings += 'public addr = %s\n\n' %osd['front_addr']

        return osd_settings

    def get_mons(self):
        return self._mons

    def get_mon(self, id=0):
        if (id >= 0):
            if id < len(self._mons):
                return self._mons[id]

    def fixup_mon_settings(self):
        # enumerate mons
        # mon_fields = ['host','mon addr']
        mon_settings = "\n"
        for mon in self.get_mons():
            mon_settings += '[mon.%s]\n' %mon['rank']
            mon_settings += 'host = %s\n' %mon['name']
            mon_settings += 'mon addr = %s\n\n' %mon['addr']

        return mon_settings

    def detect_ceph_conf(self):
        global_value_dict = {'heartbeat_interval':10,
        'osd_pool_default_size':self.ceph_report["osdmap"]['pools'][0]['size'],
        'osd_heartbeat_grace':10,
        'fsid':self.ceph_report["osdmap"]["fsid"],
        }
        global_settings = '''
[global]
heartbeat interval = %(heartbeat_interval)s
osd pool default size = %(osd_pool_default_size)s
osd heartbeat grace = %(osd_heartbeat_grace)s
keyring = /etc/ceph/keyring.admin
fsid = %(fsid)s
auth supported = cephx
        '''%(global_value_dict)
        osd_header_settings ='''
[osd]
osd mount options xfs = rw,noatime,inode64,logbsize=256k,delaylog
osd crush update on start = false
filestore xattr use omap = true
keyring = /etc/ceph/keyring.$name
osd mkfs type = %(mkfs_type)s
osd data = /var/lib/ceph/osd/osd$id
osd heartbeat interval = 10
osd heartbeat grace = 10
osd mkfs options xfs = -f
osd journal size = 0
        '''%({'mkfs_type':self.ceph_report["osd_metadata"][0].get('filestore_backend','xfs')})
        mon_header_settings ='''
[mon]
mon osd full ratio = .90
mon data = /var/lib/ceph/mon/mon$id
mon osd nearfull ratio = .75
mon clock drift allowed = .200
        '''
        osd_settings = self.fixup_osd_settings()
        mon_settings = self.fixup_mon_settings()
        cephconf = global_settings + '\n' \
                    + osd_header_settings + '\n' \
                    + osd_settings + '\n' \
                    + mon_header_settings + '\n' \
                    + mon_settings + '\n'
        return cephconf

if __name__ == '__main__':
    discover = cephconf_discover("./report.json")
    print 'osds=%s' %discover.get_osds()
    print 'osd.0=%s' %discover.get_osd(1)

    osd_settings = discover.fixup_osd_settings()
    print osd_settings
    mon_settings = discover.fixup_mon_settings()
    print mon_settings



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import webob
import json
from webob import exc
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import summary as sum_views
from vsm import conductor
from vsm import scheduler
from vsm import db
from vsm import exception
from vsm import utils

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_vsm(elem, detailed=False):
    elem.set('id')
    elem.set('name')
    elem.set('address')
    elem.set('health')
    elem.set('detail')
    elem.set('skew')
    elem.set('latency')
    elem.set('kb_total')
    elem.set('kb_used')
    elem.set('kb_avail')
    elem.set('percent_avail')

    if detailed:
        pass

vsm = make_vsm

vsm_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class VsmTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('vsm', selector='vsm')
        make_vsm(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=vsm_nsmap)

class VsmsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('vsms')
        elem = xmlutil.SubTemplateElement(root, 'vsms', selector='vsms')
        make_vsm(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=vsm_nsmap)

class Controller(wsgi.Controller):
    """The Vsm API controller for the OpenStack API."""

    def __init__(self, ext_mgr):
        super(Controller, self).__init__()

    def summary(self, req, cluster_id = None):
        LOG.info('vsm-summary.')
        context = req.environ['vsm.context']

        #TODO: as we have only one cluster for now, the cluster_id
        #has been hardcoded. In furture, client should pass
        # the cluster id by url.
        try:
            if cluster_id:
                vsm_sum = db.summary_get_by_cluster_id_and_type(context, cluster_id, 'vsm')
            else:
                vsm_sum = db.summary_get_by_type_first(context, 'vsm')
            #vsm_sum = db.summary_get_by_cluster_id_and_type(context, 1, 'vsm')
        except exception.SummaryNotFound:
            vsm_sum = None
        try:
            if cluster_id:
                ceph_sum = db.summary_get_by_cluster_id_and_type(context, cluster_id, 'ceph')
            else:
                ceph_sum = db.summary_get_by_type_first(context, 'ceph')
            #ceph_sum = db.summary_get_by_cluster_id_and_type(context, 1, 'ceph')
        except exception.SummaryNotFound:
            ceph_sum = None

        get_data = lambda x: x['summary_data'] if x else None

        sum = None
        vsm_data = get_data(vsm_sum)
        ceph_data = get_data(ceph_sum)

        if vsm_data:
            sum = json.loads(vsm_data)
            if ceph_data:
                sum.update(json.loads(ceph_data))
            else:
                sum['is_ceph_active'] = True
        elif ceph_data:
            sum = json.loads(ceph_data)
            if not vsm_data:
                sum['uptime'] = None

        if sum:
            try:
                sum['created_at'] = vsm_sum['created_at'].strftime("%Y-%m-%d %H:%M:%S")
                sum_data = {
                    'summary_data': json.dumps(sum)
                }
            except:
                pass
        else:
            sum_data = None

        LOG.info('vsm sum: %s' % sum_data)
        return sum_views.ViewBuilder().basic(sum_data, 'vsm')

def create_resource(ext_mgr):
    return wsgi.Resource(Controller(ext_mgr))




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Vsm base exception handling.

Includes decorator for re-raising Vsm-type exceptions.

SHOULD include dedicated exception logging.

"""

from oslo.config import cfg
import webob.exc

from vsm import flags
from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import log as logging

LOG = logging.getLogger(__name__)

exc_log_opts = [
    cfg.BoolOpt('fatal_exception_format_errors',
                default=False,
                help='make exception message format errors fatal'),
]

FLAGS = flags.FLAGS
FLAGS.register_opts(exc_log_opts)

class ConvertedException(webob.exc.WSGIHTTPException):
    def __init__(self, code=0, title="", explanation=""):
        self.code = code
        self.title = title
        self.explanation = explanation
        super(ConvertedException, self).__init__()

class ProcessExecutionError(IOError):
    def __init__(self, stdout=None, stderr=None, exit_code=None, cmd=None,
                 description=None):
        self.exit_code = exit_code
        self.stderr = stderr
        self.stdout = stdout
        self.cmd = cmd
        self.description = description

        if description is None:
            description = _('Unexpected error while running command.')
        if exit_code is None:
            exit_code = '-'
        message = _('%(description)s\nCommand: %(cmd)s\n'
                    'Exit code: %(exit_code)s\nStdout: %(stdout)r\n'
                    'Stderr: %(stderr)r') % locals()
        IOError.__init__(self, message)

class Error(Exception):
    pass

class DBError(Error):
    code="E-5081"
    """Wraps an implementation specific exception."""
    def __init__(self, inner_exception=None):
        self.inner_exception = inner_exception
        super(DBError, self).__init__(str(inner_exception))

def wrap_db_error(f):
    def _wrap(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except UnicodeEncodeError:
            raise InvalidUnicodeParameter()
        except Exception, e:
            LOG.exception(_('DB exception wrapped.'))
            raise DBError(e)
    _wrap.func_name = f.func_name
    return _wrap

class VsmException(Exception):
    """Base Vsm Exception

    To correctly use this class, inherit from it and define
    a 'message' property. That message will get printf'd
    with the keyword arguments provided to the constructor.

    """
    message = _("An unknown exception occurred.")
    code = 500
    headers = {}
    safe = False

    def __init__(self, message=None, **kwargs):
        self.kwargs = kwargs

        if 'code' not in self.kwargs:
            try:
                self.kwargs['code'] = self.code
            except AttributeError:
                pass

        if not message:
            try:
                message = self.message % kwargs

            except Exception as e:
                # kwargs doesn't match a variable in the message
                # log the issue and the kwargs
                LOG.exception(_('Exception in string format operation'))
                for name, value in kwargs.iteritems():
                    LOG.error("%s: %s" % (name, value))
                if FLAGS.fatal_exception_format_errors:
                    raise e
                else:
                    # at least get the core message out if something happened
                    message = self.message

        super(VsmException, self).__init__(message)

class GlanceConnectionFailed(VsmException):
    message = _("Connection to glance failed") + ": %(reason)s"

class NotAuthorized(VsmException):
    message = _("Not authorized.")
    code = 403

class AdminRequired(NotAuthorized):
    code = "E-8849"
    message = _("User does not have admin privileges")

class PolicyNotAuthorized(NotAuthorized):
    message = _("Policy doesn't allow %(action)s to be performed.")

class ImageNotAuthorized(VsmException):
    message = _("Not authorized for image %(image_id)s.")

class AppNodeNotAuthorized(NotAuthorized):
    message = _("Not authorized for Appnodes.")

class Invalid(VsmException):
    message = _("Unacceptable parameters.")
    code = 400

class InvalidSnapshot(Invalid):
    message = _("Invalid snapshot") + ": %(reason)s"

class HardwareAttached(Invalid):
    message = _("Hardware %(storage_id)s is still attached, detach storage first.")

class SfJsonEncodeFailure(VsmException):
    message = _("Failed to load data into json format")

class InvalidRequest(Invalid):
    message = _("The request is invalid.")

class InvalidResults(Invalid):
    message = _("The results are invalid.")

class InvalidInput(Invalid):
    message = _("Invalid input received") + ": %(reason)s"

class InvalidHardwareType(Invalid):
    message = _("Invalid storage type") + ": %(reason)s"

class InvalidHardware(Invalid):
    message = _("Invalid storage") + ": %(reason)s"

class InvalidPortRange(Invalid):
    message = _("Invalid port range %(from_port)s:%(to_port)s. %(msg)s")

class InvalidContentType(Invalid):
    message = _("Invalid content type %(content_type)s.")

class InvalidUnicodeParameter(Invalid):
    message = _("Invalid Parameter: "
                "Unicode is not supported by the current database.")

# Cannot be templated as the error syntax varies.
# msg needs to be constructed when raised.
class InvalidParameterValue(Invalid):
    message = _("%(err)s")

class ServiceUnavailable(Invalid):
    code = "E-86EE"
    message = _("Service is unavailable at this time.")

class VsmServiceUnavailable(ServiceUnavailable):
    code ="E-7B58"
    message = _("Vsm service is unavailable at this time.")

class ImageUnacceptable(Invalid):
    message = _("Image %(image_id)s is unacceptable: %(reason)s")

class InvalidUUID(Invalid):
    message = _("Expected a uuid but received %(uuid).")

class NotFound(VsmException):
    message = _("Resource could not be found.")
    code = 404
    safe = True

class PersistentHardwareFileNotFound(NotFound):
    message = _("Hardware %(storage_id)s persistence file could not be found.")

class HardwareNotFound(NotFound):
    message = _("Hardware %(storage_id)s could not be found.")

class SfAccountNotFound(NotFound):
    message = _("Unable to locate account %(account_name)s on "
                "Solidfire device")

class HardwareNotFoundForInstance(HardwareNotFound):
    message = _("Hardware not found for instance %(instance_id)s.")

class HardwareMetadataNotFound(NotFound):
    message = _("Hardware %(storage_id)s has no metadata with "
                "key %(metadata_key)s.")

class InvalidHardwareMetadata(Invalid):
    message = _("Invalid metadata") + ": %(reason)s"

class InvalidHardwareMetadataSize(Invalid):
    message = _("Invalid metadata size") + ": %(reason)s"

class SnapshotMetadataNotFound(NotFound):
    message = _("Snapshot %(snapshot_id)s has no metadata with "
                "key %(metadata_key)s.")

class InvalidSnapshotMetadata(Invalid):
    message = _("Invalid metadata") + ": %(reason)s"

class InvalidSnapshotMetadataSize(Invalid):
    message = _("Invalid metadata size") + ": %(reason)s"

class HardwareTypeNotFound(NotFound):
    message = _("Hardware type %(storage_type_id)s could not be found.")

class HardwareTypeNotFoundByName(HardwareTypeNotFound):
    message = _("Hardware type with name %(storage_type_name)s "
                "could not be found.")

class HardwareTypeExtraSpecsNotFound(NotFound):
    message = _("Hardware Type %(storage_type_id)s has no extra specs with "
                "key %(extra_specs_key)s.")

class SnapshotNotFound(NotFound):
    message = _("Snapshot %(snapshot_id)s could not be found.")

class HardwareIsBusy(VsmException):
    message = _("deleting storage %(storage_name)s that has snapshot")

class SnapshotIsBusy(VsmException):
    message = _("deleting snapshot %(snapshot_name)s that has "
                "dependent storages")

class ISCSITargetNotFoundForHardware(NotFound):
    message = _("No target id found for storage %(storage_id)s.")

class ISCSITargetCreateFailed(VsmException):
    message = _("Failed to create iscsi target for storage %(storage_id)s.")

class ISCSITargetAttachFailed(VsmException):
    message = _("Failed to attach iSCSI target for storage %(storage_id)s.")

class ISCSITargetRemoveFailed(VsmException):
    message = _("Failed to remove iscsi target for storage %(storage_id)s.")

class DiskNotFound(NotFound):
    code = "E-1C1A"
    message = _("No disk at %(location)s")

class InvalidImageRef(Invalid):
    message = _("Invalid image href %(image_href)s.")

class ImageNotFound(NotFound):
    message = _("Image %(image_id)s could not be found.")

class ServiceNotFound(NotFound):
    code = "E-F158"
    message = _("Service %(service_id)s could not be found.")

class HostNotFound(NotFound):
    code = "E-5B0E"
    message = _("Host %(host)s could not be found.")

#add vsm exception

class VsmHostNotFound(HostNotFound):
    code = "E-3F38"
    message = _("Vsm host %(host)s could not be found.")

class OsdNotFound(NotFound):
    code = "E-B676"
    message = _("Osd %(osd)s could not be found.")

class VsmOsdNotFound(OsdNotFound):
    code = "E-EC04"
    message = _("Osd %(osd)s could not be found.")

class VsmStorageGroupNotFound(NotFound):
    code = "E-352E"
    message = _("storagegroup %(group)% could not be found.")

#add vsm exception

class SchedulerHostFilterNotFound(NotFound):
    message = _("Scheduler Host Filter %(filter_name)s could not be found.")

class SchedulerHostWeigherNotFound(NotFound):
    message = _("Scheduler Host Weigher %(weigher_name)s could not be found.")

class HostBinaryNotFound(NotFound):
    message = _("Could not find binary %(binary)s on host %(host)s.")

class InvalidReservationExpiration(Invalid):
    message = _("Invalid reservation expiration %(expire)s.")

class InvalidQuotaValue(Invalid):
    message = _("Change would make usage less than 0 for the following "
                "resources: %(unders)s")

class QuotaNotFound(NotFound):
    message = _("Quota could not be found")

class QuotaResourceUnknown(QuotaNotFound):
    message = _("Unknown quota resources %(unknown)s.")

class ProjectQuotaNotFound(QuotaNotFound):
    message = _("Quota for project %(project_id)s could not be found.")

class QuotaClassNotFound(QuotaNotFound):
    message = _("Quota class %(class_name)s could not be found.")

class QuotaUsageNotFound(QuotaNotFound):
    message = _("Quota usage for project %(project_id)s could not be found.")

class ReservationNotFound(QuotaNotFound):
    message = _("Quota reservation %(uuid)s could not be found.")

class OverQuota(VsmException):
    message = _("Quota exceeded for resources: %(overs)s")

class MigrationNotFound(NotFound):
    message = _("Migration %(migration_id)s could not be found.")

class MigrationNotFoundByStatus(MigrationNotFound):
    message = _("Migration not found for instance %(instance_id)s "
                "with status %(status)s.")

class FileNotFound(NotFound):
    code = "E-D55E"
    message = _("File %(file_path)s could not be found.")

class ClassNotFound(NotFound):
    message = _("Class %(class_name)s could not be found: %(exception)s")

class NotAllowed(VsmException):
    message = _("Action not allowed.")

#TODO(bcwaldon): EOL this exception!
class Duplicate(VsmException):
    pass

class KeyPairExists(Duplicate):
    message = _("Key pair %(key_name)s already exists.")

class HardwareTypeExists(Duplicate):
    message = _("Hardware Type %(id)s already exists.")

class MigrationError(VsmException):
    message = _("Migration error") + ": %(reason)s"

class MalformedRequestBody(VsmException):
    message = _("Malformed message body: %(reason)s")

class ConfigNotFound(NotFound):
    code = "E-89AF"
    message = _("Could not find config at %(path)s")

class PasteAppNotFound(NotFound):
    message = _("Could not load paste app '%(name)s' from %(path)s")

class NoValidHost(VsmException):
    code = "E-6550"
    message = _("No valid host was found. %(reason)s")

class WillNotSchedule(VsmException):
    message = _("Host %(host)s is not up or doesn't exist.")

class QuotaError(VsmException):
    message = _("Quota exceeded") + ": code=%(code)s"
    code = 413
    headers = {'Retry-After': 0}
    safe = True

class HardwareSizeExceedsAvailableQuota(QuotaError):
    message = _("Requested storage or snapshot exceeds "
                "allowed Gigabytes quota")

class HardwareSizeExceedsQuota(QuotaError):
    message = _("Maximum storage/snapshot size exceeded")

class HardwareLimitExceeded(QuotaError):
    message = _("Maximum number of storages allowed (%(allowed)d) exceeded")

class SnapshotLimitExceeded(QuotaError):
    message = _("Maximum number of snapshots allowed (%(allowed)d) exceeded")

class DuplicateSfHardwareNames(Duplicate):
    message = _("Detected more than one storage with name %(vol_name)s")

class DuplicateControllerIPs(VsmException):
    code = "E-848B"
    message = _("More than one Controller IP exists: %(err)s.  %(info)s")

class Duplicate3PARHost(VsmException):
    message = _("3PAR Host already exists: %(err)s.  %(info)s")

class HardwareTypeCreateFailed(VsmException):
    message = _("Cannot create storage_type with "
                "name %(name)s and specs %(extra_specs)s")

class SolidFireAPIException(VsmException):
    message = _("Bad response from SolidFire API")

class SolidFireAPIDataException(SolidFireAPIException):
    message = _("Error in SolidFire API response: data=%(data)s")

class UnknownCmd(Invalid):
    code = "E-94FF"
    message = _("Unknown or unsupported command %(cmd)s")

class MalformedResponse(Invalid):
    message = _("Malformed response to command %(cmd)s: %(reason)s")

class BadHTTPResponseStatus(VsmException):
    code = "E-CDB5"
    message = _("Bad HTTP response status %(status)s")

class FailedCmdWithDump(VsmException):
    message = _("Operation failed with status=%(status)s. Full dump: %(data)s")

class ZadaraServerCreateFailure(VsmException):
    message = _("Unable to create server object for initiator %(name)s")

class ZadaraServerNotFound(NotFound):
    message = _("Unable to find server object for initiator %(name)s")

class ZadaraVPSANoActiveController(VsmException):
    message = _("Unable to find any active VPSA controller")

class ZadaraAttachmentsNotFound(NotFound):
    message = _("Failed to retrieve attachments for storage %(name)s")

class ZadaraInvalidAttachmentInfo(Invalid):
    message = _("Invalid attachment info for storage %(name)s: %(reason)s")

class InstanceNotFound(NotFound):
    message = _("Instance %(instance_id)s could not be found.")

class HardwareBackendAPIException(VsmException):
    message = _("Bad or unexpected response from the storage storage "
                "backend API: %(data)s")

class NfsException(VsmException):
    message = _("Unknown NFS exception")

class NfsNoSharesMounted(NotFound):
    message = _("No mounted NFS shares found")

class NfsNoSuitableShareFound(NotFound):
    message = _("There is no share which can host %(storage_size)sG")

class GlusterfsException(VsmException):
    message = _("Unknown Gluster exception")

class GlusterfsNoSharesMounted(NotFound):
    message = _("No mounted Gluster shares found")

class GlusterfsNoSuitableShareFound(NotFound):
    message = _("There is no share which can host %(storage_size)sG")

class GlanceMetadataExists(Invalid):
    message = _("Glance metadata cannot be updated, key %(key)s"
                " exists for storage id %(storage_id)s")

class ImageCopyFailure(Invalid):
    message = _("Failed to copy image to storage")

class BackupNotFound(NotFound):
    message = _("Backup %(backup_id)s could not be found.")

class InvalidBackup(Invalid):
    message = _("Invalid backup: %(reason)s")

class SwiftConnectionFailed(VsmException):
    message = _("Connection to swift failed") + ": %(reason)s"

class AppNodeNotFound(NotFound):
    message = _("App Node %(appnode_id) could not be found.")

class VsmappNotFound(NotFound):
    code = "E-AB30"
    message = _("Vsm app could not be found under the account.")

class AppNodeInvalidInfo(Invalid):
    message = _("Invalid request for App nodes.")

class DuplicateAppnode(VsmException):
    message = _("App node %(ip) already exists: %(err)s.")

class AppNodeFailure(VsmException):
    message = _("Failure on app nodes db operations") + ": %(reason)s"

class DuplicateVsmApp(VsmException):
    message = _("Vsmapp for project %(id) already exists: %(err)s.")

class DuplicateStoragePoolUsage(VsmException):
    message = _("The storage pool %(pool_id) has been presented: %(err).")

class StoragePoolUsageNotFound(NotFound):
    code = "E-1EAF"
    message = _('The storage pool usage could not be found.')

class StoragePoolUsageFailure(VsmException):
    code = "E-80F6"
    message = _('Failure on storage pool usage operations.')

class StatusTrackingError(VsmException):
    message = _('Status traceking Error.')

class StoragePoolUsageInvalid(Invalid):
    code = "E-FF52"
    message = _("Invalid request for Storage pool usages.")

class SummaryNotFound(NotFound):
    code = "E-A067"
    message = _("%(type) summary could not be found.")

class MonitorNotFound(NotFound):
    code = "E-87CA"
    message = _("monitor %(name) could not be found.")

class PGExists(VsmException):
    code = "E-9B97"
    message = _("The PG already exists.")

class PGNotFound(NotFound):
    code = "E-483842BE"
    message = _("The PG id %(pg_id) could not be found.")

class RBDExists(VsmException):
    code = "E-FC23"
    message = _("The RBD already exists.")

class RBDNotFound(NotFound):
    code = "E-567C"
    message = _("The RBD id %(rbd_id) could not be found.")

class MDSExists(VsmException):
    code = "E-6BD1"
    message = _("The MDS already exists.")

class MDSNotFound(NotFound):
    code = "E-6E62"
    message = _("The MDS id %(mds_id) could not be found.")

class ECProfileNotFound(NotFound):
    code = "E-6E1B" 
    message = _("The erasure code profile id %(ec_profile_id) could not be found.")

#    from vsm import exception
#    try:
#        raise exception.CreateStorageGroupFailed
#    except Exception, e:
#        LOG.error("%s: %s" %(e.code, e.message))

class MonitorException(VsmException):
    code = "E-A5B9"
    message = _("Monitor Exception")

class MonitorAddFailed(MonitorException):
    code = "E-47AE"
    message = _("Monitor Add Failed")

class MonitorRemoveFailed(MonitorException):
    code = "E-A4CA"
    message = _("Monitor Remove Failed")

class MonitorStartFailed(MonitorException):
    code = "E-0141"
    message = _("Monitor Start Failed")

class MonitorStopFailed(MonitorException):
    code = "E-A206"
    message = _("Monitor Stop Failed")

class StorageServerException(VsmException):
    code = "E-C79B"
    message = _("Storage Server Exception")

class StorageServerAddFailed(StorageServerException):
    code = "E-3200"
    message = _("Storage Server Add Failed")

class StorageServerRemoveFailed(StorageServerException):
    code = "E-F228"
    message = _("Storage Server Remove Failed")

class StorageServerStartFailed(StorageServerException):
    code = "E-1002"
    message = _("Storage Server Start Failed")

class StorageServerStopFailed(StorageServerException):
    code = "E-90FD"
    message = _("Storage Server Stop Failed")

class DeviceException(VsmException):
    code = "E-5682"
    message = _("Device Exception")

class DeviceAddFailed(DeviceException):
    code = "E-98E9"
    message = _("Device Add Failed")

class DeviceRemoveFailed(DeviceException):
    code = "E-7479"
    message = _("Device Remove Failed")

class DeviceStartFailed(DeviceException):
    code = "E-81C1"
    message = _("Device Start Failed")

class DeviceStopFailed(DeviceException):
    code = "E-0667"
    message = _("Device Stop Failed")

class StorageGroupException(VsmException):
    code = "E-39ED"
    message = _("Storage Group Exception")

class StorageGroupAddFailed(StorageGroupException):
    code = "E-9390"
    message = _("Storage Group Add Failed")

class PresentOpenStackException(VsmException):
    code = "E-CA97"
    message = _("Present Openstack Exception")

class AddPoolToOpenStackFailed(PresentOpenStackException):
    code = "E-3CC2"
    message = _("Add Pool To Openstack Failed")

class PathNotExist(VsmException):
    code = "E-C3C7"
    message = _("Path does't exist!")

class UpdateDBError(VsmException):
    code = "E-B869"
    message = _("Update DB error!")

class ExeCmdError(VsmException):
    code = "E-1E4C"
    message = _("Execute cmd error!")

class CephConfException(VsmException):
    code = "E-6CF6"
    message = _("Ceph config error!")

class LoadCephConfFailed(CephConfException):
    code = "E-C785"
    message = _("Load ceph config failed!")

class AddGlobalToCephConfFailed(CephConfException):
    code = "E-07C1"
    message = _("Add global item to ceph config failed!")

class AddMonToCephConfFailed(CephConfException):
    code = "E-3373"
    message = _("Add mon item to ceph config failed!")

class AddMdsToCephConfFailed(CephConfException):
    code = "E-691F"
    message = _("Add mds item to ceph config failed!")

class AddOsdToCephConfFailed(CephConfException):
    code = "E-BF96"
    message = _("Add osd item to ceph config failed!")

class GetNoneError(VsmException):
    code = "E-86A2"
    message = _("Get None error!")

class StartCephFaild(VsmException):
    code = "E-3994"
    message = _("Start ceph cluster Failed!")



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import Boolean, Column, DateTime
from sqlalchemy import Integer, MetaData, String
from sqlalchemy import Table, Index, ForeignKey
from sqlalchemy.engine.base import Engine 
from migrate.changeset.constraint import ForeignKeyConstraint
from sqlalchemy.engine import reflection
from sqlalchemy import create_engine

def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine;
    # bind migrate_engine to your metadata
    if migrate_engine.name == 'sqlite':
        return

    storage_pools = 'storage_pools'
    storage_groups = 'storage_groups'
    col = ''
    insp = reflection.Inspector.from_engine(migrate_engine)
    foreign_keys = insp.get_foreign_keys(storage_pools)
    for key in foreign_keys:
        if storage_groups == key['referred_table']:
            sql_str = "ALTER TABLE %s DROP FOREIGN KEY %s;" % (storage_pools, key['name'])
            ret = migrate_engine.execute(sql_str)
              

def downgrade(migrate_engine):
    if migrate_engine.name == 'sqlite':
        return
    #meta = MetaData()
    #meta.bind = migrate_engine

    #storage_group = Table('storage_groups',
    #                meta,
    #                autoload=True)
    #column_status = Column('status', String(255), default="OUT", nullable=False)

    try:
        #storage_group.drop_column(column_status)
        pass
    except Exception:
        raise
